@@ -1,5 +1,8 @@
 from __future__ import annotations
+
 import sys
+from pathlib import Path
+from types import ModuleType
 
 import pytest
 
@@ -15,6 +18,7 @@ from PIL import Image
 # 2.7 and 3.2.
 
 
+numpy: ModuleType | None
 try:
     import numpy
 except ImportError:
@@ -27,23 +31,24 @@ XDIM = 48000
 pytestmark = pytest.mark.skipif(sys.maxsize <= 2**32, reason="requires 64-bit system")
 
 
-def _write_png(tmp_path, xdim, ydim):
+def _write_png(tmp_path: Path, xdim: int, ydim: int) -> None:
     f = str(tmp_path / "temp.png")
     im = Image.new("L", (xdim, ydim), 0)
     im.save(f)
 
 
-def test_large(tmp_path):
+def test_large(tmp_path: Path) -> None:
     """succeeded prepatch"""
     _write_png(tmp_path, XDIM, YDIM)
 
 
-def test_2gpx(tmp_path):
+def test_2gpx(tmp_path: Path) -> None:
     """failed prepatch"""
     _write_png(tmp_path, XDIM, XDIM)
 
 
 @pytest.mark.skipif(numpy is None, reason="Numpy is not installed")
-def test_size_greater_than_int():
+def test_size_greater_than_int() -> None:
+    assert numpy is not None
     arr = numpy.ndarray(shape=(16394, 16394))
     Image.fromarray(arr)
