@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import zlib
 from io import BytesIO
 
@@ -7,7 +8,7 @@ from PIL import Image, ImageFile, PngImagePlugin
 TEST_FILE = "Tests/images/png_decompression_dos.png"
 
 
-def test_ignore_dos_text():
+def test_ignore_dos_text() -> None:
     ImageFile.LOAD_TRUNCATED_IMAGES = True
 
     try:
@@ -16,6 +17,7 @@ def test_ignore_dos_text():
     finally:
         ImageFile.LOAD_TRUNCATED_IMAGES = False
 
+    assert isinstance(im, PngImagePlugin.PngImageFile)
     for s in im.text.values():
         assert len(s) < 1024 * 1024, "Text chunk larger than 1M"
 
@@ -23,7 +25,7 @@ def test_ignore_dos_text():
         assert len(s) < 1024 * 1024, "Text chunk larger than 1M"
 
 
-def test_dos_text():
+def test_dos_text() -> None:
     try:
         im = Image.open(TEST_FILE)
         im.load()
@@ -31,11 +33,12 @@ def test_dos_text():
         assert msg, "Decompressed Data Too Large"
         return
 
+    assert isinstance(im, PngImagePlugin.PngImageFile)
     for s in im.text.values():
         assert len(s) < 1024 * 1024, "Text chunk larger than 1M"
 
 
-def test_dos_total_memory():
+def test_dos_total_memory() -> None:
     im = Image.new("L", (1, 1))
     compressed_data = zlib.compress(b"a" * 1024 * 1023)
 
@@ -52,10 +55,11 @@ def test_dos_total_memory():
     try:
         im2 = Image.open(b)
     except ValueError as msg:
-        assert "Too much memory" in msg
+        assert "Too much memory" in str(msg)
         return
 
     total_len = 0
+    assert isinstance(im2, PngImagePlugin.PngImageFile)
     for txt in im2.text.values():
         total_len += len(txt)
     assert total_len < 64 * 1024 * 1024, "Total text chunks greater than 64M"
