@@ -11,14 +11,15 @@ import subprocess
 import sys
 import sysconfig
 import tempfile
+from collections.abc import Sequence
 from functools import lru_cache
 from io import BytesIO
-from typing import Any, Callable, Sequence
+from typing import Any, Callable
 
 import pytest
 from packaging.version import parse as parse_version
 
-from PIL import Image, ImageMath, features
+from PIL import Image, ImageFile, ImageMath, features
 
 logger = logging.getLogger(__name__)
 
@@ -59,9 +60,7 @@ def convert_to_comparable(
     return new_a, new_b
 
 
-def assert_deep_equal(
-    a: Sequence[Any], b: Sequence[Any], msg: str | None = None
-) -> None:
+def assert_deep_equal(a: Any, b: Any, msg: str | None = None) -> None:
     try:
         assert len(a) == len(b), msg or f"got length {len(a)}, expected {len(b)}"
     except Exception:
@@ -174,12 +173,13 @@ def skip_unless_feature(feature: str) -> pytest.MarkDecorator:
 def skip_unless_feature_version(
     feature: str, required: str, reason: str | None = None
 ) -> pytest.MarkDecorator:
-    if not features.check(feature):
+    version = features.version(feature)
+    if version is None:
         return pytest.mark.skip(f"{feature} not available")
     if reason is None:
         reason = f"{feature} is older than {required}"
     version_required = parse_version(required)
-    version_available = parse_version(features.version(feature))
+    version_available = parse_version(version)
     return pytest.mark.skipif(version_available < version_required, reason=reason)
 
 
@@ -189,12 +189,13 @@ def mark_if_feature_version(
     version_blacklist: str,
     reason: str | None = None,
 ) -> pytest.MarkDecorator:
-    if not features.check(feature):
+    version = features.version(feature)
+    if version is None:
         return pytest.mark.pil_noop_mark()
     if reason is None:
         reason = f"{feature} is {version_blacklist}"
     version_required = parse_version(version_blacklist)
-    version_available = parse_version(features.version(feature))
+    version_available = parse_version(version)
     if (
         version_available.major == version_required.major
         and version_available.minor == version_required.minor
@@ -220,16 +221,11 @@ class PillowLeakTestCase:
         from resource import RUSAGE_SELF, getrusage
 
         mem = getrusage(RUSAGE_SELF).ru_maxrss
-        if sys.platform == "darwin":
-            # man 2 getrusage:
-            #     ru_maxrss
-            # This is the maximum resident set size utilized (in bytes).
-            return mem / 1024  # Kb
-        # linux
-        # man 2 getrusage
-        #        ru_maxrss (since Linux 2.6.32)
-        #  This is the maximum resident set size used (in kilobytes).
-        return mem  # Kb
+        # man 2 getrusage:
+        #     ru_maxrss
+        # This is the maximum resident set size utilized
+        # in bytes on macOS, in kilobytes on Linux
+        return mem / 1024 if sys.platform == "darwin" else mem
 
     def _test_leak(self, core: Callable[[], None]) -> None:
         start_mem = self._get_mem_usage()
@@ -243,7 +239,7 @@ class PillowLeakTestCase:
 # helpers
 
 
-def fromstring(data: bytes) -> Image.Image:
+def fromstring(data: bytes) -> ImageFile.ImageFile:
     return Image.open(BytesIO(data))
 
 
@@ -273,7 +269,18 @@ def _cached_hopper(mode: str) -> Image.Image:
         im = hopper("L")
     else:
         im = hopper()
-    return im.convert(mode)
+    if mode.startswith("BGR;"):
+        with pytest.warns(DeprecationWarning):
+            im = im.convert(mode)
+    else:
+        try:
+            im = im.convert(mode)
+        except ImportError:
+            if mode == "LAB":
+                im = Image.open("Tests/images/hopper.Lab.tif")
+            else:
+                raise
+    return im
 
 
 def djpeg_available() -> bool:
