@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import subprocess
 import sys
 
@@ -6,13 +7,14 @@ import fuzzers
 import packaging
 import pytest
 
-from PIL import Image, features
+from PIL import Image, UnidentifiedImageError, features
 from Tests.helper import skip_unless_feature
 
 if sys.platform.startswith("win32"):
     pytest.skip("Fuzzer is linux only", allow_module_level=True)
-if features.check("libjpeg_turbo"):
-    version = packaging.version.parse(features.version("libjpeg_turbo"))
+libjpeg_turbo_version = features.version("libjpeg_turbo")
+if libjpeg_turbo_version is not None:
+    version = packaging.version.parse(libjpeg_turbo_version)
     if version.major == 2 and version.minor == 0:
         pytestmark = pytest.mark.valgrind_known_error(
             reason="Known failing with libjpeg_turbo 2.0"
@@ -23,7 +25,7 @@ if features.check("libjpeg_turbo"):
     "path",
     subprocess.check_output("find Tests/images -type f", shell=True).split(b"\n"),
 )
-def test_fuzz_images(path):
+def test_fuzz_images(path: str) -> None:
     fuzzers.enable_decompressionbomb_error()
     try:
         with open(path, "rb") as f:
@@ -42,7 +44,7 @@ def test_fuzz_images(path):
     except (
         Image.DecompressionBombError,
         Image.DecompressionBombWarning,
-        Image.UnidentifiedImageError,
+        UnidentifiedImageError,
     ):
         # Known Image.* exceptions
         assert True
@@ -54,7 +56,7 @@ def test_fuzz_images(path):
 @pytest.mark.parametrize(
     "path", subprocess.check_output("find Tests/fonts -type f", shell=True).split(b"\n")
 )
-def test_fuzz_fonts(path):
+def test_fuzz_fonts(path: str) -> None:
     if not path:
         return
     with open(path, "rb") as f:
