@@ -1,14 +1,18 @@
 from __future__ import annotations
+
+from pathlib import Path
+from typing import IO
+
 import pytest
 
-from PIL import BufrStubImagePlugin, Image
+from PIL import BufrStubImagePlugin, Image, ImageFile
 
 from .helper import hopper
 
 TEST_FILE = "Tests/images/gfs.t06z.rassda.tm00.bufr_d"
 
 
-def test_open():
+def test_open() -> None:
     # Act
     with Image.open(TEST_FILE) as im:
         # Assert
@@ -19,7 +23,7 @@ def test_open():
         assert im.size == (1, 1)
 
 
-def test_invalid_file():
+def test_invalid_file() -> None:
     # Arrange
     invalid_file = "Tests/images/flower.jpg"
 
@@ -28,7 +32,7 @@ def test_invalid_file():
         BufrStubImagePlugin.BufrStubImageFile(invalid_file)
 
 
-def test_load():
+def test_load() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         # Act / Assert: stub cannot load without an implemented handler
@@ -36,7 +40,7 @@ def test_load():
             im.load()
 
 
-def test_save(tmp_path):
+def test_save(tmp_path: Path) -> None:
     # Arrange
     im = hopper()
     tmpfile = str(tmp_path / "temp.bufr")
@@ -46,34 +50,37 @@ def test_save(tmp_path):
         im.save(tmpfile)
 
 
-def test_handler(tmp_path):
-    class TestHandler:
+def test_handler(tmp_path: Path) -> None:
+    class TestHandler(ImageFile.StubHandler):
         opened = False
         loaded = False
         saved = False
 
-        def open(self, im):
+        def open(self, im: ImageFile.StubImageFile) -> None:
             self.opened = True
 
-        def load(self, im):
+        def load(self, im: ImageFile.StubImageFile) -> Image.Image:
             self.loaded = True
             im.fp.close()
             return Image.new("RGB", (1, 1))
 
-        def save(self, im, fp, filename):
+        def is_loaded(self) -> bool:
+            return self.loaded
+
+        def save(self, im: Image.Image, fp: IO[bytes], filename: str) -> None:
             self.saved = True
 
     handler = TestHandler()
     BufrStubImagePlugin.register_handler(handler)
     with Image.open(TEST_FILE) as im:
         assert handler.opened
-        assert not handler.loaded
+        assert not handler.is_loaded()
 
         im.load()
-        assert handler.loaded
+        assert handler.is_loaded()
 
         temp_file = str(tmp_path / "temp.bufr")
         im.save(temp_file)
         assert handler.saved
 
-    BufrStubImagePlugin._handler = None
+    BufrStubImagePlugin.register_handler(None)
