@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import warnings
 
 import pytest
@@ -11,7 +12,7 @@ from .helper import assert_image_equal, hopper, is_pypy
 TEST_FILE = "Tests/images/hopper.dcx"
 
 
-def test_sanity():
+def test_sanity() -> None:
     # Arrange
 
     # Act
@@ -24,8 +25,8 @@ def test_sanity():
 
 
 @pytest.mark.skipif(is_pypy(), reason="Requires CPython")
-def test_unclosed_file():
-    def open():
+def test_unclosed_file() -> None:
+    def open() -> None:
         im = Image.open(TEST_FILE)
         im.load()
 
@@ -33,26 +34,30 @@ def test_unclosed_file():
         open()
 
 
-def test_closed_file():
+def test_closed_file() -> None:
     with warnings.catch_warnings():
+        warnings.simplefilter("error")
+
         im = Image.open(TEST_FILE)
         im.load()
         im.close()
 
 
-def test_context_manager():
+def test_context_manager() -> None:
     with warnings.catch_warnings():
+        warnings.simplefilter("error")
+
         with Image.open(TEST_FILE) as im:
             im.load()
 
 
-def test_invalid_file():
+def test_invalid_file() -> None:
     with open("Tests/images/flower.jpg", "rb") as fp:
         with pytest.raises(SyntaxError):
             DcxImagePlugin.DcxImageFile(fp)
 
 
-def test_tell():
+def test_tell() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         # Act
@@ -62,13 +67,13 @@ def test_tell():
         assert frame == 0
 
 
-def test_n_frames():
+def test_n_frames() -> None:
     with Image.open(TEST_FILE) as im:
         assert im.n_frames == 1
         assert not im.is_animated
 
 
-def test_eoferror():
+def test_eoferror() -> None:
     with Image.open(TEST_FILE) as im:
         n_frames = im.n_frames
 
@@ -81,7 +86,7 @@ def test_eoferror():
         im.seek(n_frames - 1)
 
 
-def test_seek_too_far():
+def test_seek_too_far() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         frame = 999  # too big on purpose
