@@ -8,6 +8,7 @@ import pytest
 from PIL import EpsImagePlugin, Image, UnidentifiedImageError, features
 
 from .helper import (
+    assert_image_equal_tofile,
     assert_image_similar,
     assert_image_similar_tofile,
     hopper,
@@ -19,18 +20,18 @@ from .helper import (
 HAS_GHOSTSCRIPT = EpsImagePlugin.has_ghostscript()
 
 # Our two EPS test files (they are identical except for their bounding boxes)
-FILE1 = "Tests/images/zero_bb.eps"
-FILE2 = "Tests/images/non_zero_bb.eps"
+FILE1 = "Tests/images/eps/zero_bb.eps"
+FILE2 = "Tests/images/eps/non_zero_bb.eps"
 
 # Due to palletization, we'll need to convert these to RGB after load
-FILE1_COMPARE = "Tests/images/zero_bb.png"
-FILE1_COMPARE_SCALE2 = "Tests/images/zero_bb_scale2.png"
+FILE1_COMPARE = "Tests/images/eps/zero_bb.png"
+FILE1_COMPARE_SCALE2 = "Tests/images/eps/zero_bb_scale2.png"
 
-FILE2_COMPARE = "Tests/images/non_zero_bb.png"
-FILE2_COMPARE_SCALE2 = "Tests/images/non_zero_bb_scale2.png"
+FILE2_COMPARE = "Tests/images/eps/non_zero_bb.png"
+FILE2_COMPARE_SCALE2 = "Tests/images/eps/non_zero_bb_scale2.png"
 
 # EPS test files with binary preview
-FILE3 = "Tests/images/binary_preview_map.eps"
+FILE3 = "Tests/images/eps/binary_preview_map.eps"
 
 # Three unsigned 32bit little-endian values:
 #   0xC6D3D0C5 magic number
@@ -80,9 +81,7 @@ simple_eps_file_with_long_binary_data = (
 
 
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
-@pytest.mark.parametrize(
-    ("filename", "size"), ((FILE1, (460, 352)), (FILE2, (360, 252)))
-)
+@pytest.mark.parametrize("filename, size", ((FILE1, (460, 352)), (FILE2, (360, 252))))
 @pytest.mark.parametrize("scale", (1, 2))
 def test_sanity(filename: str, size: tuple[int, int], scale: int) -> None:
     expected_size = tuple(s * scale for s in size)
@@ -128,6 +127,15 @@ def test_binary_header_only() -> None:
         EpsImagePlugin.EpsImageFile(data)
 
 
+@pytest.mark.parametrize("prefix", (b"", simple_binary_header))
+def test_simple_eps_file(prefix: bytes) -> None:
+    data = io.BytesIO(prefix + b"\n".join(simple_eps_file))
+    with Image.open(data) as img:
+        assert img.mode == "RGB"
+        assert img.size == (100, 100)
+        assert img.format == "EPS"
+
+
 @pytest.mark.parametrize("prefix", (b"", simple_binary_header))
 def test_missing_version_comment(prefix: bytes) -> None:
     data = io.BytesIO(prefix + b"\n".join(simple_eps_file_without_version))
@@ -143,23 +151,21 @@ def test_missing_boundingbox_comment(prefix: bytes) -> None:
 
 
 @pytest.mark.parametrize("prefix", (b"", simple_binary_header))
-def test_invalid_boundingbox_comment(prefix: bytes) -> None:
-    data = io.BytesIO(prefix + b"\n".join(simple_eps_file_with_invalid_boundingbox))
+@pytest.mark.parametrize(
+    "file_lines",
+    (
+        simple_eps_file_with_invalid_boundingbox,
+        simple_eps_file_with_invalid_boundingbox_valid_imagedata,
+    ),
+)
+def test_invalid_boundingbox_comment(
+    prefix: bytes, file_lines: tuple[bytes, ...]
+) -> None:
+    data = io.BytesIO(prefix + b"\n".join(file_lines))
     with pytest.raises(OSError, match="cannot determine EPS bounding box"):
         EpsImagePlugin.EpsImageFile(data)
 
 
-@pytest.mark.parametrize("prefix", (b"", simple_binary_header))
-def test_invalid_boundingbox_comment_valid_imagedata_comment(prefix: bytes) -> None:
-    data = io.BytesIO(
-        prefix + b"\n".join(simple_eps_file_with_invalid_boundingbox_valid_imagedata)
-    )
-    with Image.open(data) as img:
-        assert img.mode == "RGB"
-        assert img.size == (100, 100)
-        assert img.format == "EPS"
-
-
 @pytest.mark.parametrize("prefix", (b"", simple_binary_header))
 def test_ascii_comment_too_long(prefix: bytes) -> None:
     data = io.BytesIO(prefix + b"\n".join(simple_eps_file_with_long_ascii_comment))
@@ -179,7 +185,7 @@ def test_load_long_binary_data(prefix: bytes) -> None:
     data = io.BytesIO(prefix + b"\n".join(simple_eps_file_with_long_binary_data))
     with Image.open(data) as img:
         img.load()
-        assert img.mode == "RGB"
+        assert img.mode == "1"
         assert img.size == (100, 100)
         assert img.format == "EPS"
 
@@ -189,7 +195,7 @@ def test_load_long_binary_data(prefix: bytes) -> None:
 )
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
 def test_cmyk() -> None:
-    with Image.open("Tests/images/pil_sample_cmyk.eps") as cmyk_image:
+    with Image.open("Tests/images/eps/pil_sample_cmyk.eps") as cmyk_image:
         assert cmyk_image.mode == "CMYK"
         assert cmyk_image.size == (100, 100)
         assert cmyk_image.format == "EPS"
@@ -206,8 +212,8 @@ def test_cmyk() -> None:
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
 def test_showpage() -> None:
     # See https://github.com/python-pillow/Pillow/issues/2615
-    with Image.open("Tests/images/reqd_showpage.eps") as plot_image:
-        with Image.open("Tests/images/reqd_showpage.png") as target:
+    with Image.open("Tests/images/eps/reqd_showpage.eps") as plot_image:
+        with Image.open("Tests/images/eps/reqd_showpage.png") as target:
             # should not crash/hang
             plot_image.load()
             # fonts could be slightly different
@@ -216,11 +222,11 @@ def test_showpage() -> None:
 
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
 def test_transparency() -> None:
-    with Image.open("Tests/images/reqd_showpage.eps") as plot_image:
+    with Image.open("Tests/images/eps/reqd_showpage.eps") as plot_image:
         plot_image.load(transparency=True)
         assert plot_image.mode == "RGBA"
 
-        with Image.open("Tests/images/reqd_showpage_transparency.png") as target:
+        with Image.open("Tests/images/eps/reqd_showpage_transparency.png") as target:
             # fonts could be slightly different
             assert_image_similar(plot_image, target, 6)
 
@@ -247,9 +253,19 @@ def test_bytesio_object() -> None:
         assert_image_similar(img, image1_scale1_compare, 5)
 
 
-def test_1_mode() -> None:
-    with Image.open("Tests/images/1.eps") as im:
-        assert im.mode == "1"
+@pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
+@pytest.mark.parametrize(
+    # These images have an "ImageData" descriptor.
+    "filename",
+    (
+        "Tests/images/eps/1.eps",
+        "Tests/images/eps/1_boundingbox_after_imagedata.eps",
+        "Tests/images/eps/1_second_imagedata.eps",
+    ),
+)
+def test_1(filename: str) -> None:
+    with Image.open(filename) as im:
+        assert_image_equal_tofile(im, "Tests/images/eps/1.bmp")
 
 
 def test_image_mode_not_supported(tmp_path: Path) -> None:
@@ -304,7 +320,9 @@ def test_render_scale2() -> None:
 
 
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
-@pytest.mark.parametrize("filename", (FILE1, FILE2, "Tests/images/illu10_preview.eps"))
+@pytest.mark.parametrize(
+    "filename", (FILE1, FILE2, "Tests/images/eps/illu10_preview.eps")
+)
 def test_resize(filename: str) -> None:
     with Image.open(filename) as im:
         new_size = (100, 100)
@@ -329,46 +347,6 @@ def test_read_binary_preview() -> None:
         pass
 
 
-def test_readline_psfile(tmp_path: Path) -> None:
-    # check all the freaking line endings possible from the spec
-    # test_string = u'something\r\nelse\n\rbaz\rbif\n'
-    line_endings = ["\r\n", "\n", "\n\r", "\r"]
-    strings = ["something", "else", "baz", "bif"]
-
-    def _test_readline(t: EpsImagePlugin.PSFile, ending: str) -> None:
-        ending = f"Failure with line ending: {''.join(str(ord(s)) for s in ending)}"
-        assert t.readline().strip("\r\n") == "something", ending
-        assert t.readline().strip("\r\n") == "else", ending
-        assert t.readline().strip("\r\n") == "baz", ending
-        assert t.readline().strip("\r\n") == "bif", ending
-
-    def _test_readline_io_psfile(test_string: str, ending: str) -> None:
-        f = io.BytesIO(test_string.encode("latin-1"))
-        with pytest.warns(DeprecationWarning):
-            t = EpsImagePlugin.PSFile(f)
-        _test_readline(t, ending)
-
-    def _test_readline_file_psfile(test_string: str, ending: str) -> None:
-        f = str(tmp_path / "temp.txt")
-        with open(f, "wb") as w:
-            w.write(test_string.encode("latin-1"))
-
-        with open(f, "rb") as r:
-            with pytest.warns(DeprecationWarning):
-                t = EpsImagePlugin.PSFile(r)
-            _test_readline(t, ending)
-
-    for ending in line_endings:
-        s = ending.join(strings)
-        _test_readline_io_psfile(s, ending)
-        _test_readline_file_psfile(s, ending)
-
-
-def test_psfile_deprecation() -> None:
-    with pytest.warns(DeprecationWarning):
-        EpsImagePlugin.PSFile(None)
-
-
 @pytest.mark.parametrize("prefix", (b"", simple_binary_header))
 @pytest.mark.parametrize(
     "line_ending",
@@ -386,10 +364,10 @@ def test_readline(prefix: bytes, line_ending: bytes) -> None:
 @pytest.mark.parametrize(
     "filename",
     (
-        "Tests/images/illu10_no_preview.eps",
-        "Tests/images/illu10_preview.eps",
-        "Tests/images/illuCS6_no_preview.eps",
-        "Tests/images/illuCS6_preview.eps",
+        "Tests/images/eps/illu10_no_preview.eps",
+        "Tests/images/eps/illu10_preview.eps",
+        "Tests/images/eps/illuCS6_no_preview.eps",
+        "Tests/images/eps/illuCS6_preview.eps",
     ),
 )
 def test_open_eps(filename: str) -> None:
@@ -401,7 +379,7 @@ def test_open_eps(filename: str) -> None:
 @pytest.mark.skipif(not HAS_GHOSTSCRIPT, reason="Ghostscript not available")
 def test_emptyline() -> None:
     # Test file includes an empty line in the header data
-    emptyline_file = "Tests/images/zero_bb_emptyline.eps"
+    emptyline_file = "Tests/images/eps/zero_bb_emptyline.eps"
 
     with Image.open(emptyline_file) as image:
         image.load()
@@ -413,7 +391,7 @@ def test_emptyline() -> None:
 @pytest.mark.timeout(timeout=5)
 @pytest.mark.parametrize(
     "test_file",
-    ["Tests/images/timeout-d675703545fee17acab56e5fec644c19979175de.eps"],
+    ["Tests/images/eps/timeout-d675703545fee17acab56e5fec644c19979175de.eps"],
 )
 def test_timeout(test_file: str) -> None:
     with open(test_file, "rb") as f:
@@ -425,20 +403,21 @@ def test_timeout(test_file: str) -> None:
 def test_bounding_box_in_trailer() -> None:
     # Check bounding boxes are parsed in the same way
     # when specified in the header and the trailer
-    with Image.open("Tests/images/zero_bb_trailer.eps") as trailer_image, Image.open(
-        FILE1
-    ) as header_image:
+    with (
+        Image.open("Tests/images/eps/zero_bb_trailer.eps") as trailer_image,
+        Image.open(FILE1) as header_image,
+    ):
         assert trailer_image.size == header_image.size
 
 
 def test_eof_before_bounding_box() -> None:
     with pytest.raises(OSError):
-        with Image.open("Tests/images/zero_bb_eof_before_boundingbox.eps"):
+        with Image.open("Tests/images/eps/zero_bb_eof_before_boundingbox.eps"):
             pass
 
 
 def test_invalid_data_after_eof() -> None:
-    with open("Tests/images/illuCS6_preview.eps", "rb") as f:
+    with open("Tests/images/eps/illuCS6_preview.eps", "rb") as f:
         img_bytes = io.BytesIO(f.read() + b"\r\n%" + (b" " * 255))
 
     with Image.open(img_bytes) as img:
