@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import pytest
 
 from PIL import GdImageFile, UnidentifiedImageError
@@ -6,18 +7,18 @@ from PIL import GdImageFile, UnidentifiedImageError
 TEST_GD_FILE = "Tests/images/hopper.gd"
 
 
-def test_sanity():
+def test_sanity() -> None:
     with GdImageFile.open(TEST_GD_FILE) as im:
         assert im.size == (128, 128)
         assert im.format == "GD"
 
 
-def test_bad_mode():
+def test_bad_mode() -> None:
     with pytest.raises(ValueError):
         GdImageFile.open(TEST_GD_FILE, "bad mode")
 
 
-def test_invalid_file():
+def test_invalid_file() -> None:
     invalid_file = "Tests/images/flower.jpg"
 
     with pytest.raises(UnidentifiedImageError):
