@@ -1,10 +1,11 @@
 from __future__ import annotations
+
 import pytest
 
 from PIL.GimpPaletteFile import GimpPaletteFile
 
 
-def test_sanity():
+def test_sanity() -> None:
     with open("Tests/images/test.gpl", "rb") as fp:
         GimpPaletteFile(fp)
 
@@ -21,7 +22,7 @@ def test_sanity():
             GimpPaletteFile(fp)
 
 
-def test_get_palette():
+def test_get_palette() -> None:
     # Arrange
     with open("Tests/images/custom_gimp_palette.gpl", "rb") as fp:
         palette_file = GimpPaletteFile(fp)
