@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import sys
 from io import BytesIO, StringIO
 
@@ -11,7 +12,7 @@ from .helper import assert_image_equal, hopper
 TEST_FILE = "Tests/images/iptc.jpg"
 
 
-def test_open():
+def test_open() -> None:
     expected = Image.new("L", (1, 1))
 
     f = BytesIO(
@@ -23,7 +24,7 @@ def test_open():
         assert_image_equal(im, expected)
 
 
-def test_getiptcinfo_jpg_none():
+def test_getiptcinfo_jpg_none() -> None:
     # Arrange
     with hopper() as im:
         # Act
@@ -33,7 +34,7 @@ def test_getiptcinfo_jpg_none():
     assert iptc is None
 
 
-def test_getiptcinfo_jpg_found():
+def test_getiptcinfo_jpg_found() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         # Act
@@ -45,7 +46,7 @@ def test_getiptcinfo_jpg_found():
     assert iptc[(2, 101)] == b"Hungary"
 
 
-def test_getiptcinfo_fotostation():
+def test_getiptcinfo_fotostation() -> None:
     # Arrange
     with open(TEST_FILE, "rb") as fp:
         data = bytearray(fp.read())
@@ -56,13 +57,14 @@ def test_getiptcinfo_fotostation():
         iptc = IptcImagePlugin.getiptcinfo(im)
 
     # Assert
+    assert iptc is not None
     for tag in iptc.keys():
         if tag[0] == 240:
             return
     pytest.fail("FotoStation tag not found")
 
 
-def test_getiptcinfo_zero_padding():
+def test_getiptcinfo_zero_padding() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         im.info["photoshop"][0x0404] += b"\x00\x00\x00"
@@ -75,7 +77,17 @@ def test_getiptcinfo_zero_padding():
     assert len(iptc) == 3
 
 
-def test_getiptcinfo_tiff_none():
+def test_getiptcinfo_tiff() -> None:
+    # Arrange
+    with Image.open("Tests/images/hopper.Lab.tif") as im:
+        # Act
+        iptc = IptcImagePlugin.getiptcinfo(im)
+
+    # Assert
+    assert iptc == {(1, 90): b"\x1b%G", (2, 0): b"\xcf\xc0"}
+
+
+def test_getiptcinfo_tiff_none() -> None:
     # Arrange
     with Image.open("Tests/images/hopper.tif") as im:
         # Act
@@ -85,7 +97,7 @@ def test_getiptcinfo_tiff_none():
     assert iptc is None
 
 
-def test_i():
+def test_i() -> None:
     # Arrange
     c = b"a"
 
@@ -97,7 +109,7 @@ def test_i():
     assert ret == 97
 
 
-def test_dump(monkeypatch):
+def test_dump(monkeypatch: pytest.MonkeyPatch) -> None:
     # Arrange
     c = b"abc"
     # Temporarily redirect stdout
@@ -112,6 +124,6 @@ def test_dump(monkeypatch):
     assert mystdout.getvalue() == "61 62 63 \n"
 
 
-def test_pad_deprecation():
+def test_pad_deprecation() -> None:
     with pytest.warns(DeprecationWarning):
         assert IptcImagePlugin.PAD == b"\0\0\0\0"
