@@ -1,5 +1,7 @@
 from __future__ import annotations
+
 import os
+from pathlib import Path
 
 import pytest
 
@@ -12,7 +14,7 @@ EXTRA_DIR = "Tests/images/picins"
 YA_EXTRA_DIR = "Tests/images/msp"
 
 
-def test_sanity(tmp_path):
+def test_sanity(tmp_path: Path) -> None:
     test_file = str(tmp_path / "temp.msp")
 
     hopper("1").save(test_file)
@@ -24,14 +26,14 @@ def test_sanity(tmp_path):
         assert im.format == "MSP"
 
 
-def test_invalid_file():
+def test_invalid_file() -> None:
     invalid_file = "Tests/images/flower.jpg"
 
     with pytest.raises(SyntaxError):
         MspImagePlugin.MspImageFile(invalid_file)
 
 
-def test_bad_checksum():
+def test_bad_checksum() -> None:
     # Arrange
     # This was created by forcing Pillow to save with checksum=0
     bad_checksum = "Tests/images/hopper_bad_checksum.msp"
@@ -41,7 +43,7 @@ def test_bad_checksum():
         MspImagePlugin.MspImageFile(bad_checksum)
 
 
-def test_open_windows_v1():
+def test_open_windows_v1() -> None:
     # Arrange
     # Act
     with Image.open(TEST_FILE) as im:
@@ -50,7 +52,7 @@ def test_open_windows_v1():
         assert isinstance(im, MspImagePlugin.MspImageFile)
 
 
-def _assert_file_image_equal(source_path, target_path):
+def _assert_file_image_equal(source_path: str, target_path: str) -> None:
     with Image.open(source_path) as im:
         assert_image_equal_tofile(im, target_path)
 
@@ -58,7 +60,7 @@ def _assert_file_image_equal(source_path, target_path):
 @pytest.mark.skipif(
     not os.path.exists(EXTRA_DIR), reason="Extra image files not installed"
 )
-def test_open_windows_v2():
+def test_open_windows_v2() -> None:
     files = (
         os.path.join(EXTRA_DIR, f)
         for f in os.listdir(EXTRA_DIR)
@@ -71,7 +73,7 @@ def test_open_windows_v2():
 @pytest.mark.skipif(
     not os.path.exists(YA_EXTRA_DIR), reason="Even More Extra image files not installed"
 )
-def test_msp_v2():
+def test_msp_v2() -> None:
     for f in os.listdir(YA_EXTRA_DIR):
         if ".MSP" not in f:
             continue
@@ -79,7 +81,7 @@ def test_msp_v2():
         _assert_file_image_equal(path, path.replace(".MSP", ".png"))
 
 
-def test_cannot_save_wrong_mode(tmp_path):
+def test_cannot_save_wrong_mode(tmp_path: Path) -> None:
     # Arrange
     im = hopper()
     filename = str(tmp_path / "temp.msp")
