@@ -1,9 +1,13 @@
 from __future__ import annotations
+
 import re
 import sys
 import warnings
 import zlib
 from io import BytesIO
+from pathlib import Path
+from types import ModuleType
+from typing import Any, cast
 
 import pytest
 
@@ -20,6 +24,7 @@ from .helper import (
     skip_unless_feature,
 )
 
+ElementTree: ModuleType | None
 try:
     from defusedxml import ElementTree
 except ImportError:
@@ -34,9 +39,9 @@ TEST_PNG_FILE = "Tests/images/hopper.png"
 MAGIC = PngImagePlugin._MAGIC
 
 
-def chunk(cid, *data):
+def chunk(cid: bytes, *data: bytes) -> bytes:
     test_file = BytesIO()
-    PngImagePlugin.putchunk(*(test_file, cid) + data)
+    PngImagePlugin.putchunk(test_file, cid, *data)
     return test_file.getvalue()
 
 
@@ -50,20 +55,20 @@ HEAD = MAGIC + IHDR
 TAIL = IDAT + IEND
 
 
-def load(data):
+def load(data: bytes) -> Image.Image:
     return Image.open(BytesIO(data))
 
 
-def roundtrip(im, **options):
+def roundtrip(im: Image.Image, **options: Any) -> PngImagePlugin.PngImageFile:
     out = BytesIO()
     im.save(out, "PNG", **options)
     out.seek(0)
-    return Image.open(out)
+    return cast(PngImagePlugin.PngImageFile, Image.open(out))
 
 
 @skip_unless_feature("zlib")
 class TestFilePng:
-    def get_chunks(self, filename):
+    def get_chunks(self, filename: str) -> list[bytes]:
         chunks = []
         with open(filename, "rb") as fp:
             fp.read(8)
@@ -78,9 +83,11 @@ class TestFilePng:
                     png.crc(cid, s)
         return chunks
 
-    def test_sanity(self, tmp_path):
+    def test_sanity(self, tmp_path: Path) -> None:
         # internal version number
-        assert re.search(r"\d+(\.\d+){1,3}$", features.version_codec("zlib"))
+        version = features.version_codec("zlib")
+        assert version is not None
+        assert re.search(r"\d+(\.\d+){1,3}(\.zlib\-ng)?$", version)
 
         test_file = str(tmp_path / "temp.png")
 
@@ -97,17 +104,17 @@ class TestFilePng:
             im = hopper(mode)
             im.save(test_file)
             with Image.open(test_file) as reloaded:
-                if mode in ("I;16", "I;16B"):
+                if mode in ("I", "I;16B"):
                     reloaded = reloaded.convert(mode)
                 assert_image_equal(reloaded, im)
 
-    def test_invalid_file(self):
+    def test_invalid_file(self) -> None:
         invalid_file = "Tests/images/flower.jpg"
 
         with pytest.raises(SyntaxError):
             PngImagePlugin.PngImageFile(invalid_file)
 
-    def test_broken(self):
+    def test_broken(self) -> None:
         # Check reading of totally broken files.  In this case, the test
         # file was checked into Subversion as a text file.
 
@@ -116,7 +123,7 @@ class TestFilePng:
             with Image.open(test_file):
                 pass
 
-    def test_bad_text(self):
+    def test_bad_text(self) -> None:
         # Make sure PIL can read malformed tEXt chunks (@PIL152)
 
         im = load(HEAD + chunk(b"tEXt") + TAIL)
@@ -134,7 +141,7 @@ class TestFilePng:
         im = load(HEAD + chunk(b"tEXt", b"spam\0egg\0") + TAIL)
         assert im.info == {"spam": "egg\x00"}
 
-    def test_bad_ztxt(self):
+    def test_bad_ztxt(self) -> None:
         # Test reading malformed zTXt chunks (python-pillow/Pillow#318)
 
         im = load(HEAD + chunk(b"zTXt") + TAIL)
@@ -155,7 +162,7 @@ class TestFilePng:
         im = load(HEAD + chunk(b"zTXt", b"spam\0\0" + zlib.compress(b"egg")) + TAIL)
         assert im.info == {"spam": "egg"}
 
-    def test_bad_itxt(self):
+    def test_bad_itxt(self) -> None:
         im = load(HEAD + chunk(b"iTXt") + TAIL)
         assert im.info == {}
 
@@ -199,7 +206,7 @@ class TestFilePng:
         assert im.info["spam"].lang == "en"
         assert im.info["spam"].tkey == "Spam"
 
-    def test_interlace(self):
+    def test_interlace(self) -> None:
         test_file = "Tests/images/pil123p.png"
         with Image.open(test_file) as im:
             assert_image(im, "P", (162, 150))
@@ -214,7 +221,7 @@ class TestFilePng:
 
             im.load()
 
-    def test_load_transparent_p(self):
+    def test_load_transparent_p(self) -> None:
         test_file = "Tests/images/pil123p.png"
         with Image.open(test_file) as im:
             assert_image(im, "P", (162, 150))
@@ -224,7 +231,7 @@ class TestFilePng:
         # image has 124 unique alpha values
         assert len(im.getchannel("A").getcolors()) == 124
 
-    def test_load_transparent_rgb(self):
+    def test_load_transparent_rgb(self) -> None:
         test_file = "Tests/images/rgb_trns.png"
         with Image.open(test_file) as im:
             assert im.info["transparency"] == (0, 255, 52)
@@ -236,7 +243,7 @@ class TestFilePng:
         # image has 876 transparent pixels
         assert im.getchannel("A").getcolors()[0][0] == 876
 
-    def test_save_p_transparent_palette(self, tmp_path):
+    def test_save_p_transparent_palette(self, tmp_path: Path) -> None:
         in_file = "Tests/images/pil123p.png"
         with Image.open(in_file) as im:
             # 'transparency' contains a byte string with the opacity for
@@ -257,7 +264,7 @@ class TestFilePng:
         # image has 124 unique alpha values
         assert len(im.getchannel("A").getcolors()) == 124
 
-    def test_save_p_single_transparency(self, tmp_path):
+    def test_save_p_single_transparency(self, tmp_path: Path) -> None:
         in_file = "Tests/images/p_trns_single.png"
         with Image.open(in_file) as im:
             # pixel value 164 is full transparent
@@ -280,7 +287,7 @@ class TestFilePng:
         # image has 876 transparent pixels
         assert im.getchannel("A").getcolors()[0][0] == 876
 
-    def test_save_p_transparent_black(self, tmp_path):
+    def test_save_p_transparent_black(self, tmp_path: Path) -> None:
         # check if solid black image with full transparency
         # is supported (check for #1838)
         im = Image.new("RGBA", (10, 10), (0, 0, 0, 0))
@@ -298,9 +305,9 @@ class TestFilePng:
         assert_image(im, "RGBA", (10, 10))
         assert im.getcolors() == [(100, (0, 0, 0, 0))]
 
-    def test_save_grayscale_transparency(self, tmp_path):
-        for mode, num_transparent in {"1": 1994, "L": 559, "I": 559}.items():
-            in_file = "Tests/images/" + mode.lower() + "_trns.png"
+    def test_save_grayscale_transparency(self, tmp_path: Path) -> None:
+        for mode, num_transparent in {"1": 1994, "L": 559, "I;16": 559}.items():
+            in_file = "Tests/images/" + mode.split(";")[0].lower() + "_trns.png"
             with Image.open(in_file) as im:
                 assert im.mode == mode
                 assert im.info["transparency"] == 255
@@ -319,18 +326,20 @@ class TestFilePng:
             test_im_rgba = test_im.convert("RGBA")
             assert test_im_rgba.getchannel("A").getcolors()[0][0] == num_transparent
 
-    def test_save_rgb_single_transparency(self, tmp_path):
+    def test_save_rgb_single_transparency(self, tmp_path: Path) -> None:
         in_file = "Tests/images/caption_6_33_22.png"
         with Image.open(in_file) as im:
             test_file = str(tmp_path / "temp.png")
             im.save(test_file)
 
-    def test_load_verify(self):
+    def test_load_verify(self) -> None:
         # Check open/load/verify exception (@PIL150)
 
         with Image.open(TEST_PNG_FILE) as im:
             # Assert that there is no unclosed file warning
             with warnings.catch_warnings():
+                warnings.simplefilter("error")
+
                 im.verify()
 
         with Image.open(TEST_PNG_FILE) as im:
@@ -338,7 +347,7 @@ class TestFilePng:
             with pytest.raises(RuntimeError):
                 im.verify()
 
-    def test_verify_struct_error(self):
+    def test_verify_struct_error(self) -> None:
         # Check open/load/verify exception (#1755)
 
         # offsets to test, -10: breaks in i32() in read. (OSError)
@@ -354,7 +363,7 @@ class TestFilePng:
                 with pytest.raises((OSError, SyntaxError)):
                     im.verify()
 
-    def test_verify_ignores_crc_error(self):
+    def test_verify_ignores_crc_error(self) -> None:
         # check ignores crc errors in ancillary chunks
 
         chunk_data = chunk(b"tEXt", b"spam")
@@ -371,7 +380,7 @@ class TestFilePng:
         finally:
             ImageFile.LOAD_TRUNCATED_IMAGES = False
 
-    def test_verify_not_ignores_crc_error_in_required_chunk(self):
+    def test_verify_not_ignores_crc_error_in_required_chunk(self) -> None:
         # check does not ignore crc errors in required chunks
 
         image_data = MAGIC + IHDR[:-1] + b"q" + TAIL
@@ -383,18 +392,18 @@ class TestFilePng:
         finally:
             ImageFile.LOAD_TRUNCATED_IMAGES = False
 
-    def test_roundtrip_dpi(self):
+    def test_roundtrip_dpi(self) -> None:
         # Check dpi roundtripping
 
         with Image.open(TEST_PNG_FILE) as im:
             im = roundtrip(im, dpi=(100.33, 100.33))
         assert im.info["dpi"] == (100.33, 100.33)
 
-    def test_load_float_dpi(self):
+    def test_load_float_dpi(self) -> None:
         with Image.open(TEST_PNG_FILE) as im:
             assert im.info["dpi"] == (95.9866, 95.9866)
 
-    def test_roundtrip_text(self):
+    def test_roundtrip_text(self) -> None:
         # Check text roundtripping
 
         with Image.open(TEST_PNG_FILE) as im:
@@ -406,7 +415,7 @@ class TestFilePng:
         assert im.info == {"TXT": "VALUE", "ZIP": "VALUE"}
         assert im.text == {"TXT": "VALUE", "ZIP": "VALUE"}
 
-    def test_roundtrip_itxt(self):
+    def test_roundtrip_itxt(self) -> None:
         # Check iTXt roundtripping
 
         im = Image.new("RGB", (32, 32))
@@ -417,12 +426,14 @@ class TestFilePng:
         im = roundtrip(im, pnginfo=info)
         assert im.info == {"spam": "Eggs", "eggs": "Spam"}
         assert im.text == {"spam": "Eggs", "eggs": "Spam"}
+        assert isinstance(im.text["spam"], PngImagePlugin.iTXt)
         assert im.text["spam"].lang == "en"
         assert im.text["spam"].tkey == "Spam"
+        assert isinstance(im.text["eggs"], PngImagePlugin.iTXt)
         assert im.text["eggs"].lang == "en"
         assert im.text["eggs"].tkey == "Eggs"
 
-    def test_nonunicode_text(self):
+    def test_nonunicode_text(self) -> None:
         # Check so that non-Unicode text is saved as a tEXt rather than iTXt
 
         im = Image.new("RGB", (32, 32))
@@ -431,10 +442,10 @@ class TestFilePng:
         im = roundtrip(im, pnginfo=info)
         assert isinstance(im.info["Text"], str)
 
-    def test_unicode_text(self):
+    def test_unicode_text(self) -> None:
         # Check preservation of non-ASCII characters
 
-        def rt_text(value):
+        def rt_text(value: str) -> None:
             im = Image.new("RGB", (32, 32))
             info = PngImagePlugin.PngInfo()
             info.add_text("Text", value)
@@ -447,7 +458,7 @@ class TestFilePng:
         rt_text(chr(0x4E00) + chr(0x66F0) + chr(0x9FBA) + chr(0x3042) + chr(0xAC00))
         rt_text("A" + chr(0xC4) + chr(0x472) + chr(0x3042))  # Combined
 
-    def test_scary(self):
+    def test_scary(self) -> None:
         # Check reading of evil PNG file.  For information, see:
         # http://scary.beasts.org/security/CESA-2004-001.txt
         # The first byte is removed from pngtest_bad.png
@@ -461,7 +472,7 @@ class TestFilePng:
             with Image.open(pngfile):
                 pass
 
-    def test_trns_rgb(self):
+    def test_trns_rgb(self) -> None:
         # Check writing and reading of tRNS chunks for RGB images.
         # Independent file sample provided by Sebastian Spaeth.
 
@@ -476,7 +487,7 @@ class TestFilePng:
         im = roundtrip(im, transparency=(0, 1, 2))
         assert im.info["transparency"] == (0, 1, 2)
 
-    def test_trns_p(self, tmp_path):
+    def test_trns_p(self, tmp_path: Path) -> None:
         # Check writing a transparency of 0, issue #528
         im = hopper("P")
         im.info["transparency"] = 0
@@ -489,13 +500,13 @@ class TestFilePng:
 
             assert_image_equal(im2.convert("RGBA"), im.convert("RGBA"))
 
-    def test_trns_null(self):
+    def test_trns_null(self) -> None:
         # Check reading images with null tRNS value, issue #1239
         test_file = "Tests/images/tRNS_null_1x1.png"
         with Image.open(test_file) as im:
             assert im.info["transparency"] == 0
 
-    def test_save_icc_profile(self):
+    def test_save_icc_profile(self) -> None:
         with Image.open("Tests/images/icc_profile_none.png") as im:
             assert im.info["icc_profile"] is None
 
@@ -505,40 +516,42 @@ class TestFilePng:
                 im = roundtrip(im, icc_profile=expected_icc)
                 assert im.info["icc_profile"] == expected_icc
 
-    def test_discard_icc_profile(self):
+    def test_discard_icc_profile(self) -> None:
         with Image.open("Tests/images/icc_profile.png") as im:
             assert "icc_profile" in im.info
 
             im = roundtrip(im, icc_profile=None)
         assert "icc_profile" not in im.info
 
-    def test_roundtrip_icc_profile(self):
+    def test_roundtrip_icc_profile(self) -> None:
         with Image.open("Tests/images/icc_profile.png") as im:
             expected_icc = im.info["icc_profile"]
 
             im = roundtrip(im)
         assert im.info["icc_profile"] == expected_icc
 
-    def test_roundtrip_no_icc_profile(self):
+    def test_roundtrip_no_icc_profile(self) -> None:
         with Image.open("Tests/images/icc_profile_none.png") as im:
             assert im.info["icc_profile"] is None
 
             im = roundtrip(im)
         assert "icc_profile" not in im.info
 
-    def test_repr_png(self):
+    def test_repr_png(self) -> None:
         im = hopper()
+        b = im._repr_png_()
+        assert b is not None
 
-        with Image.open(BytesIO(im._repr_png_())) as repr_png:
+        with Image.open(BytesIO(b)) as repr_png:
             assert repr_png.format == "PNG"
             assert_image_equal(im, repr_png)
 
-    def test_repr_png_error_returns_none(self):
+    def test_repr_png_error_returns_none(self) -> None:
         im = hopper("F")
 
         assert im._repr_png_() is None
 
-    def test_chunk_order(self, tmp_path):
+    def test_chunk_order(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/icc_profile.png") as im:
             test_file = str(tmp_path / "temp.png")
             im.convert("P").save(test_file, dpi=(100, 100))
@@ -559,17 +572,17 @@ class TestFilePng:
         # pHYs - before IDAT
         assert chunks.index(b"pHYs") < chunks.index(b"IDAT")
 
-    def test_getchunks(self):
+    def test_getchunks(self) -> None:
         im = hopper()
 
         chunks = PngImagePlugin.getchunks(im)
         assert len(chunks) == 3
 
-    def test_read_private_chunks(self):
+    def test_read_private_chunks(self) -> None:
         with Image.open("Tests/images/exif.png") as im:
             assert im.private_chunks == [(b"orNT", b"\x01")]
 
-    def test_roundtrip_private_chunk(self):
+    def test_roundtrip_private_chunk(self) -> None:
         # Check private chunk roundtripping
 
         with Image.open(TEST_PNG_FILE) as im:
@@ -587,7 +600,7 @@ class TestFilePng:
             (b"prIV", b"VALUE3", True),
         ]
 
-    def test_textual_chunks_after_idat(self):
+    def test_textual_chunks_after_idat(self) -> None:
         with Image.open("Tests/images/hopper.png") as im:
             assert "comment" in im.text
             for k, v in {
@@ -614,7 +627,11 @@ class TestFilePng:
         with Image.open("Tests/images/hopper_idat_after_image_end.png") as im:
             assert im.text == {"TXT": "VALUE", "ZIP": "VALUE"}
 
-    def test_padded_idat(self):
+    def test_unknown_compression_method(self) -> None:
+        with pytest.raises(SyntaxError, match="Unknown compression method"):
+            PngImagePlugin.PngImageFile("Tests/images/unknown_compression_method.png")
+
+    def test_padded_idat(self) -> None:
         # This image has been manually hexedited
         # so that the IDAT chunk has padding at the end
         # Set MAXBLOCK to the length of the actual data
@@ -634,7 +651,7 @@ class TestFilePng:
     @pytest.mark.parametrize(
         "cid", (b"IHDR", b"sRGB", b"pHYs", b"acTL", b"fcTL", b"fdAT")
     )
-    def test_truncated_chunks(self, cid):
+    def test_truncated_chunks(self, cid: bytes) -> None:
         fp = BytesIO()
         with PngImagePlugin.PngStream(fp) as png:
             with pytest.raises(ValueError):
@@ -644,16 +661,17 @@ class TestFilePng:
             png.call(cid, 0, 0)
             ImageFile.LOAD_TRUNCATED_IMAGES = False
 
-    def test_specify_bits(self, tmp_path):
+    @pytest.mark.parametrize("save_all", (True, False))
+    def test_specify_bits(self, save_all: bool, tmp_path: Path) -> None:
         im = hopper("P")
 
         out = str(tmp_path / "temp.png")
-        im.save(out, bits=4)
+        im.save(out, bits=4, save_all=save_all)
 
         with Image.open(out) as reloaded:
             assert len(reloaded.png.im_palette[1]) == 48
 
-    def test_plte_length(self, tmp_path):
+    def test_plte_length(self, tmp_path: Path) -> None:
         im = Image.new("P", (1, 1))
         im.putpalette((1, 1, 1))
 
@@ -663,7 +681,7 @@ class TestFilePng:
         with Image.open(out) as reloaded:
             assert len(reloaded.png.im_palette[1]) == 3
 
-    def test_getxmp(self):
+    def test_getxmp(self) -> None:
         with Image.open("Tests/images/color_snakes.png") as im:
             if ElementTree is None:
                 with pytest.warns(
@@ -672,13 +690,14 @@ class TestFilePng:
                 ):
                     assert im.getxmp() == {}
             else:
+                assert "xmp" in im.info
                 xmp = im.getxmp()
 
                 description = xmp["xmpmeta"]["RDF"]["Description"]
                 assert description["PixelXDimension"] == "10"
                 assert description["subject"]["Seq"] is None
 
-    def test_exif(self):
+    def test_exif(self) -> None:
         # With an EXIF chunk
         with Image.open("Tests/images/exif.png") as im:
             exif = im._getexif()
@@ -704,7 +723,7 @@ class TestFilePng:
             exif = im.getexif()
         assert exif[274] == 3
 
-    def test_exif_save(self, tmp_path):
+    def test_exif_save(self, tmp_path: Path) -> None:
         # Test exif is not saved from info
         test_file = str(tmp_path / "temp.png")
         with Image.open("Tests/images/exif.png") as im:
@@ -724,7 +743,7 @@ class TestFilePng:
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_exif_from_jpg(self, tmp_path):
+    def test_exif_from_jpg(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/pil_sample_rgb.jpg") as im:
             test_file = str(tmp_path / "temp.png")
             im.save(test_file, exif=im.getexif())
@@ -733,7 +752,7 @@ class TestFilePng:
             exif = reloaded._getexif()
         assert exif[305] == "Adobe Photoshop CS Macintosh"
 
-    def test_exif_argument(self, tmp_path):
+    def test_exif_argument(self, tmp_path: Path) -> None:
         with Image.open(TEST_PNG_FILE) as im:
             test_file = str(tmp_path / "temp.png")
             im.save(test_file, exif=b"exifstring")
@@ -741,11 +760,11 @@ class TestFilePng:
         with Image.open(test_file) as reloaded:
             assert reloaded.info["exif"] == b"Exif\x00\x00exifstring"
 
-    def test_tell(self):
+    def test_tell(self) -> None:
         with Image.open(TEST_PNG_FILE) as im:
             assert im.tell() == 0
 
-    def test_seek(self):
+    def test_seek(self) -> None:
         with Image.open(TEST_PNG_FILE) as im:
             im.seek(0)
 
@@ -753,31 +772,35 @@ class TestFilePng:
                 im.seek(1)
 
     @pytest.mark.parametrize("buffer", (True, False))
-    def test_save_stdout(self, buffer):
-        old_stdout = sys.stdout
+    def test_save_stdout(self, buffer: bool, monkeypatch: pytest.MonkeyPatch) -> None:
 
-        if buffer:
+        class MyStdOut:
+            buffer = BytesIO()
 
-            class MyStdOut:
-                buffer = BytesIO()
+        mystdout: MyStdOut | BytesIO = MyStdOut() if buffer else BytesIO()
 
-            mystdout = MyStdOut()
-        else:
-            mystdout = BytesIO()
-
-        sys.stdout = mystdout
+        monkeypatch.setattr(sys, "stdout", mystdout)
 
         with Image.open(TEST_PNG_FILE) as im:
             im.save(sys.stdout, "PNG")
 
-        # Reset stdout
-        sys.stdout = old_stdout
-
-        if buffer:
+        if isinstance(mystdout, MyStdOut):
             mystdout = mystdout.buffer
         with Image.open(mystdout) as reloaded:
             assert_image_equal_tofile(reloaded, TEST_PNG_FILE)
 
+    def test_truncated_end_chunk(self) -> None:
+        with Image.open("Tests/images/truncated_end_chunk.png") as im:
+            with pytest.raises(OSError):
+                im.load()
+
+        ImageFile.LOAD_TRUNCATED_IMAGES = True
+        try:
+            with Image.open("Tests/images/truncated_end_chunk.png") as im:
+                assert_image_equal_tofile(im, "Tests/images/hopper.png")
+        finally:
+            ImageFile.LOAD_TRUNCATED_IMAGES = False
+
 
 @pytest.mark.skipif(is_win32(), reason="Requires Unix or macOS")
 @skip_unless_feature("zlib")
@@ -785,7 +808,7 @@ class TestTruncatedPngPLeaks(PillowLeakTestCase):
     mem_limit = 2 * 1024  # max increase in K
     iterations = 100  # Leak is 56k/iteration, this will leak 5.6megs
 
-    def test_leak_load(self):
+    def test_leak_load(self) -> None:
         with open("Tests/images/hopper.png", "rb") as f:
             DATA = BytesIO(f.read(16 * 1024))
 
@@ -793,7 +816,7 @@ class TestTruncatedPngPLeaks(PillowLeakTestCase):
         with Image.open(DATA) as im:
             im.load()
 
-        def core():
+        def core() -> None:
             with Image.open(DATA) as im:
                 im.load()
 
