@@ -1,11 +1,13 @@
 from __future__ import annotations
+
 import os
 from glob import glob
 from itertools import product
+from pathlib import Path
 
 import pytest
 
-from PIL import Image
+from PIL import Image, UnidentifiedImageError
 
 from .helper import assert_image_equal, assert_image_equal_tofile, hopper
 
@@ -20,8 +22,8 @@ _ORIGIN_TO_ORIENTATION = {"tl": 1, "bl": -1}
 
 
 @pytest.mark.parametrize("mode", _MODES)
-def test_sanity(mode, tmp_path):
-    def roundtrip(original_im):
+def test_sanity(mode: str, tmp_path: Path) -> None:
+    def roundtrip(original_im: Image.Image) -> None:
         out = str(tmp_path / "temp.tga")
 
         original_im.save(out, rle=rle)
@@ -63,17 +65,31 @@ def test_sanity(mode, tmp_path):
                     roundtrip(original_im)
 
 
-def test_palette_depth_16(tmp_path):
+def test_palette_depth_8(tmp_path: Path) -> None:
+    with pytest.raises(UnidentifiedImageError):
+        Image.open("Tests/images/p_8.tga")
+
+
+def test_palette_depth_16(tmp_path: Path) -> None:
     with Image.open("Tests/images/p_16.tga") as im:
-        assert_image_equal_tofile(im.convert("RGB"), "Tests/images/p_16.png")
+        assert im.palette.mode == "RGBA"
+        assert_image_equal_tofile(im.convert("RGBA"), "Tests/images/p_16.png")
 
         out = str(tmp_path / "temp.png")
         im.save(out)
         with Image.open(out) as reloaded:
-            assert_image_equal_tofile(reloaded.convert("RGB"), "Tests/images/p_16.png")
+            assert_image_equal_tofile(reloaded.convert("RGBA"), "Tests/images/p_16.png")
 
 
-def test_id_field():
+def test_rgba_16() -> None:
+    with Image.open("Tests/images/rgba16.tga") as im:
+        assert im.mode == "RGBA"
+
+        assert im.getpixel((0, 0)) == (172, 0, 255, 255)
+        assert im.getpixel((1, 0)) == (0, 255, 82, 0)
+
+
+def test_id_field() -> None:
     # tga file with id field
     test_file = "Tests/images/tga_id_field.tga"
 
@@ -83,7 +99,7 @@ def test_id_field():
         assert im.size == (100, 100)
 
 
-def test_id_field_rle():
+def test_id_field_rle() -> None:
     # tga file with id field
     test_file = "Tests/images/rgb32rle.tga"
 
@@ -93,7 +109,7 @@ def test_id_field_rle():
         assert im.size == (199, 199)
 
 
-def test_cross_scan_line():
+def test_cross_scan_line() -> None:
     with Image.open("Tests/images/cross_scan_line.tga") as im:
         assert_image_equal_tofile(im, "Tests/images/cross_scan_line.png")
 
@@ -102,7 +118,7 @@ def test_cross_scan_line():
             im.load()
 
 
-def test_save(tmp_path):
+def test_save(tmp_path: Path) -> None:
     test_file = "Tests/images/tga_id_field.tga"
     with Image.open(test_file) as im:
         out = str(tmp_path / "temp.tga")
@@ -119,7 +135,7 @@ def test_save(tmp_path):
         assert test_im.size == (100, 100)
 
 
-def test_small_palette(tmp_path):
+def test_small_palette(tmp_path: Path) -> None:
     im = Image.new("P", (1, 1))
     colors = [0, 0, 0]
     im.putpalette(colors)
@@ -131,7 +147,12 @@ def test_small_palette(tmp_path):
         assert reloaded.getpalette() == colors
 
 
-def test_save_wrong_mode(tmp_path):
+def test_missing_palette() -> None:
+    with Image.open("Tests/images/dilation4.lut") as im:
+        assert im.mode == "L"
+
+
+def test_save_wrong_mode(tmp_path: Path) -> None:
     im = hopper("PA")
     out = str(tmp_path / "temp.tga")
 
@@ -139,7 +160,7 @@ def test_save_wrong_mode(tmp_path):
         im.save(out)
 
 
-def test_save_mapdepth():
+def test_save_mapdepth() -> None:
     # This image has been manually hexedited from 200x32_p_bl_raw.tga
     # to include an origin
     test_file = "Tests/images/200x32_p_bl_raw_origin.tga"
@@ -147,7 +168,7 @@ def test_save_mapdepth():
         assert_image_equal_tofile(im, "Tests/images/tga/common/200x32_p.png")
 
 
-def test_save_id_section(tmp_path):
+def test_save_id_section(tmp_path: Path) -> None:
     test_file = "Tests/images/rgb32rle.tga"
     with Image.open(test_file) as im:
         out = str(tmp_path / "temp.tga")
@@ -178,7 +199,7 @@ def test_save_id_section(tmp_path):
         assert "id_section" not in test_im.info
 
 
-def test_save_orientation(tmp_path):
+def test_save_orientation(tmp_path: Path) -> None:
     test_file = "Tests/images/rgb32rle.tga"
     out = str(tmp_path / "temp.tga")
     with Image.open(test_file) as im:
@@ -189,7 +210,7 @@ def test_save_orientation(tmp_path):
         assert test_im.info["orientation"] == 1
 
 
-def test_horizontal_orientations():
+def test_horizontal_orientations() -> None:
     # These images have been manually hexedited to have the relevant orientations
     with Image.open("Tests/images/rgb32rle_top_right.tga") as im:
         assert im.load()[90, 90][:3] == (0, 0, 0)
@@ -198,7 +219,7 @@ def test_horizontal_orientations():
         assert im.load()[90, 90][:3] == (0, 255, 0)
 
 
-def test_save_rle(tmp_path):
+def test_save_rle(tmp_path: Path) -> None:
     test_file = "Tests/images/rgb32rle.tga"
     with Image.open(test_file) as im:
         assert im.info["compression"] == "tga_rle"
@@ -231,7 +252,7 @@ def test_save_rle(tmp_path):
         assert test_im.info["compression"] == "tga_rle"
 
 
-def test_save_l_transparency(tmp_path):
+def test_save_l_transparency(tmp_path: Path) -> None:
     # There are 559 transparent pixels in la.tga.
     num_transparent = 559
 
