@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import io
 import logging
 import os
@@ -6,6 +7,9 @@ import shutil
 import sys
 import tempfile
 import warnings
+from pathlib import Path
+from types import ModuleType
+from typing import IO, Any
 
 import pytest
 
@@ -13,6 +17,7 @@ from PIL import (
     ExifTags,
     Image,
     ImageDraw,
+    ImageFile,
     ImagePalette,
     UnidentifiedImageError,
     features,
@@ -21,57 +26,53 @@ from PIL import (
 from .helper import (
     assert_image_equal,
     assert_image_equal_tofile,
+    assert_image_similar,
     assert_image_similar_tofile,
     assert_not_all_same,
     hopper,
+    is_big_endian,
     is_win32,
     mark_if_feature_version,
     skip_unless_feature,
 )
 
+ElementTree: ModuleType | None
+try:
+    from defusedxml import ElementTree
+except ImportError:
+    ElementTree = None
+
+PrettyPrinter: type | None
+try:
+    from IPython.lib.pretty import PrettyPrinter
+except ImportError:
+    PrettyPrinter = None
+
+
+# Deprecation helper
+def helper_image_new(mode: str, size: tuple[int, int]) -> Image.Image:
+    if mode.startswith("BGR;"):
+        with pytest.warns(DeprecationWarning):
+            return Image.new(mode, size)
+    else:
+        return Image.new(mode, size)
+
 
 class TestImage:
-    @pytest.mark.parametrize(
-        "mode",
-        (
-            "1",
-            "P",
-            "PA",
-            "L",
-            "LA",
-            "La",
-            "F",
-            "I",
-            "I;16",
-            "I;16L",
-            "I;16B",
-            "I;16N",
-            "RGB",
-            "RGBX",
-            "RGBA",
-            "RGBa",
-            "BGR;15",
-            "BGR;16",
-            "BGR;24",
-            "CMYK",
-            "YCbCr",
-            "LAB",
-            "HSV",
-        ),
-    )
-    def test_image_modes_success(self, mode):
-        Image.new(mode, (1, 1))
+    @pytest.mark.parametrize("mode", Image.MODES + ["BGR;15", "BGR;16", "BGR;24"])
+    def test_image_modes_success(self, mode: str) -> None:
+        helper_image_new(mode, (1, 1))
 
     @pytest.mark.parametrize("mode", ("", "bad", "very very long"))
-    def test_image_modes_fail(self, mode):
+    def test_image_modes_fail(self, mode: str) -> None:
         with pytest.raises(ValueError) as e:
             Image.new(mode, (1, 1))
         assert str(e.value) == "unrecognized image mode"
 
-    def test_exception_inheritance(self):
+    def test_exception_inheritance(self) -> None:
         assert issubclass(UnidentifiedImageError, OSError)
 
-    def test_sanity(self):
+    def test_sanity(self) -> None:
         im = Image.new("L", (100, 100))
         assert repr(im)[:45] == "<PIL.Image.Image image mode=L size=100x100 at"
         assert im.mode == "L"
@@ -96,26 +97,33 @@ class TestImage:
         # with pytest.raises(MemoryError):
         #   Image.new("L", (1000000, 1000000))
 
-    def test_repr_pretty(self):
-        class Pretty:
-            def text(self, text):
-                self.pretty_output = text
-
+    @pytest.mark.skipif(PrettyPrinter is None, reason="IPython is not installed")
+    def test_repr_pretty(self) -> None:
         im = Image.new("L", (100, 100))
 
-        p = Pretty()
-        im._repr_pretty_(p, None)
-        assert p.pretty_output == "<PIL.Image.Image image mode=L size=100x100>"
+        output = io.StringIO()
+        assert PrettyPrinter is not None
+        p = PrettyPrinter(output)
+        im._repr_pretty_(p, False)
+        assert output.getvalue() == "<PIL.Image.Image image mode=L size=100x100>"
 
-    def test_open_formats(self):
+    def test_open_formats(self) -> None:
         PNGFILE = "Tests/images/hopper.png"
         JPGFILE = "Tests/images/hopper.jpg"
 
         with pytest.raises(TypeError):
-            with Image.open(PNGFILE, formats=123):
+            with Image.open(PNGFILE, formats=123):  # type: ignore[arg-type]
                 pass
 
-        for formats in [["JPEG"], ("JPEG",), ["jpeg"], ["Jpeg"], ["jPeG"], ["JpEg"]]:
+        format_list: list[list[str] | tuple[str, ...]] = [
+            ["JPEG"],
+            ("JPEG",),
+            ["jpeg"],
+            ["Jpeg"],
+            ["jPeG"],
+            ["JpEg"],
+        ]
+        for formats in format_list:
             with pytest.raises(UnidentifiedImageError):
                 with Image.open(PNGFILE, formats=formats):
                     pass
@@ -129,39 +137,46 @@ class TestImage:
                 assert im.mode == "RGB"
                 assert im.size == (128, 128)
 
-    def test_width_height(self):
+    def test_open_verbose_failure(self, monkeypatch: pytest.MonkeyPatch) -> None:
+        monkeypatch.setattr(Image, "WARN_POSSIBLE_FORMATS", True)
+
+        im = io.BytesIO(b"")
+        with pytest.warns(UserWarning):
+            with pytest.raises(UnidentifiedImageError):
+                with Image.open(im):
+                    pass
+
+    def test_width_height(self) -> None:
         im = Image.new("RGB", (1, 2))
         assert im.width == 1
         assert im.height == 2
 
         with pytest.raises(AttributeError):
-            im.size = (3, 4)
+            im.size = (3, 4)  # type: ignore[misc]
 
-    def test_set_mode(self):
+    def test_set_mode(self) -> None:
         im = Image.new("RGB", (1, 1))
 
         with pytest.raises(AttributeError):
-            im.mode = "P"
+            im.mode = "P"  # type: ignore[misc]
 
-    def test_invalid_image(self):
+    def test_invalid_image(self) -> None:
         im = io.BytesIO(b"")
         with pytest.raises(UnidentifiedImageError):
             with Image.open(im):
                 pass
 
-    def test_bad_mode(self):
+    def test_bad_mode(self) -> None:
         with pytest.raises(ValueError):
-            with Image.open("filename", "bad mode"):
+            with Image.open("filename", "bad mode"):  # type: ignore[arg-type]
                 pass
 
-    def test_stringio(self):
+    def test_stringio(self) -> None:
         with pytest.raises(ValueError):
-            with Image.open(io.StringIO()):
+            with Image.open(io.StringIO()):  # type: ignore[arg-type]
                 pass
 
-    def test_pathlib(self, tmp_path):
-        from PIL.Image import Path
-
+    def test_pathlib(self, tmp_path: Path) -> None:
         with Image.open(Path("Tests/images/multipage-mmap.tiff")) as im:
             assert im.mode == "P"
             assert im.size == (10, 10)
@@ -178,12 +193,22 @@ class TestImage:
                     os.remove(temp_file)
                 im.save(Path(temp_file))
 
-    def test_fp_name(self, tmp_path):
+    def test_fp_name(self, tmp_path: Path) -> None:
         temp_file = str(tmp_path / "temp.jpg")
 
-        class FP:
-            def write(self, b):
-                pass
+        class FP(io.BytesIO):
+            name: str
+
+            if sys.version_info >= (3, 12):
+                from collections.abc import Buffer
+
+                def write(self, data: Buffer) -> int:
+                    return len(data)
+
+            else:
+
+                def write(self, data: Any) -> int:
+                    return len(data)
 
         fp = FP()
         fp.name = temp_file
@@ -191,22 +216,23 @@ class TestImage:
         im = hopper()
         im.save(fp)
 
-    def test_tempfile(self):
+    def test_tempfile(self) -> None:
         # see #1460, pathlib support breaks tempfile.TemporaryFile on py27
         # Will error out on save on 3.0.0
         im = hopper()
         with tempfile.TemporaryFile() as fp:
             im.save(fp, "JPEG")
             fp.seek(0)
-            assert_image_similar_tofile(im, fp, 20)
+            with Image.open(fp) as reloaded:
+                assert_image_similar(im, reloaded, 20)
 
-    def test_unknown_extension(self, tmp_path):
+    def test_unknown_extension(self, tmp_path: Path) -> None:
         im = hopper()
         temp_file = str(tmp_path / "temp.unknown")
         with pytest.raises(ValueError):
             im.save(temp_file)
 
-    def test_internals(self):
+    def test_internals(self) -> None:
         im = Image.new("L", (100, 100))
         im.readonly = 1
         im._copy()
@@ -221,7 +247,7 @@ class TestImage:
         sys.platform == "cygwin",
         reason="Test requires opening an mmaped file for writing",
     )
-    def test_readonly_save(self, tmp_path):
+    def test_readonly_save(self, tmp_path: Path) -> None:
         temp_file = str(tmp_path / "temp.bmp")
         shutil.copy("Tests/images/rgb32bf-rgba.bmp", temp_file)
 
@@ -229,7 +255,7 @@ class TestImage:
             assert im.readonly
             im.save(temp_file)
 
-    def test_dump(self, tmp_path):
+    def test_dump(self, tmp_path: Path) -> None:
         im = Image.new("L", (10, 10))
         im._dump(str(tmp_path / "temp_L.ppm"))
 
@@ -240,7 +266,7 @@ class TestImage:
         with pytest.raises(ValueError):
             im._dump(str(tmp_path / "temp_HSV.ppm"))
 
-    def test_comparison_with_other_type(self):
+    def test_comparison_with_other_type(self) -> None:
         # Arrange
         item = Image.new("RGB", (25, 25), "#000")
         num = 12
@@ -250,7 +276,7 @@ class TestImage:
         assert item is not None
         assert item != num
 
-    def test_expand_x(self):
+    def test_expand_x(self) -> None:
         # Arrange
         im = hopper()
         orig_size = im.size
@@ -263,7 +289,7 @@ class TestImage:
         assert im.size[0] == orig_size[0] + 2 * xmargin
         assert im.size[1] == orig_size[1] + 2 * xmargin
 
-    def test_expand_xy(self):
+    def test_expand_xy(self) -> None:
         # Arrange
         im = hopper()
         orig_size = im.size
@@ -277,12 +303,12 @@ class TestImage:
         assert im.size[0] == orig_size[0] + 2 * xmargin
         assert im.size[1] == orig_size[1] + 2 * ymargin
 
-    def test_getbands(self):
+    def test_getbands(self) -> None:
         # Assert
         assert hopper("RGB").getbands() == ("R", "G", "B")
         assert hopper("YCbCr").getbands() == ("Y", "Cb", "Cr")
 
-    def test_getchannel_wrong_params(self):
+    def test_getchannel_wrong_params(self) -> None:
         im = hopper()
 
         with pytest.raises(ValueError):
@@ -294,7 +320,7 @@ class TestImage:
         with pytest.raises(ValueError):
             im.getchannel("1")
 
-    def test_getchannel(self):
+    def test_getchannel(self) -> None:
         im = hopper("YCbCr")
         Y, Cb, Cr = im.split()
 
@@ -305,7 +331,7 @@ class TestImage:
         assert_image_equal(Cr, im.getchannel(2))
         assert_image_equal(Cr, im.getchannel("Cr"))
 
-    def test_getbbox(self):
+    def test_getbbox(self) -> None:
         # Arrange
         im = hopper()
 
@@ -315,7 +341,7 @@ class TestImage:
         # Assert
         assert bbox == (0, 0, 128, 128)
 
-    def test_ne(self):
+    def test_ne(self) -> None:
         # Arrange
         im1 = Image.new("RGB", (25, 25), "black")
         im2 = Image.new("RGB", (25, 25), "white")
@@ -323,7 +349,7 @@ class TestImage:
         # Act / Assert
         assert im1 != im2
 
-    def test_alpha_composite(self):
+    def test_alpha_composite(self) -> None:
         # https://stackoverflow.com/questions/3374878
         # Arrange
         expected_colors = sorted(
@@ -351,10 +377,11 @@ class TestImage:
         img = Image.alpha_composite(dst, src)
 
         # Assert
-        img_colors = sorted(img.getcolors())
-        assert img_colors == expected_colors
+        img_colors = img.getcolors()
+        assert img_colors is not None
+        assert sorted(img_colors) == expected_colors
 
-    def test_alpha_inplace(self):
+    def test_alpha_inplace(self) -> None:
         src = Image.new("RGBA", (128, 128), "blue")
 
         over = Image.new("RGBA", (128, 128), "red")
@@ -396,17 +423,17 @@ class TestImage:
 
         # errors
         with pytest.raises(ValueError):
-            source.alpha_composite(over, "invalid source")
+            source.alpha_composite(over, "invalid destination")  # type: ignore[arg-type]
         with pytest.raises(ValueError):
-            source.alpha_composite(over, (0, 0), "invalid destination")
+            source.alpha_composite(over, (0, 0), "invalid source")  # type: ignore[arg-type]
         with pytest.raises(ValueError):
-            source.alpha_composite(over, 0)
+            source.alpha_composite(over, 0)  # type: ignore[arg-type]
         with pytest.raises(ValueError):
-            source.alpha_composite(over, (0, 0), 0)
+            source.alpha_composite(over, (0, 0), 0)  # type: ignore[arg-type]
         with pytest.raises(ValueError):
             source.alpha_composite(over, (0, 0), (0, -1))
 
-    def test_register_open_duplicates(self):
+    def test_register_open_duplicates(self) -> None:
         # Arrange
         factory, accept = Image.OPEN["JPEG"]
         id_length = len(Image.ID)
@@ -417,7 +444,7 @@ class TestImage:
         # Assert
         assert len(Image.ID) == id_length
 
-    def test_registered_extensions_uninitialized(self):
+    def test_registered_extensions_uninitialized(self) -> None:
         # Arrange
         Image._initialized = 0
 
@@ -427,7 +454,7 @@ class TestImage:
         # Assert
         assert Image._initialized == 2
 
-    def test_registered_extensions(self):
+    def test_registered_extensions(self) -> None:
         # Arrange
         # Open an image to trigger plugin registration
         with Image.open("Tests/images/rgb.jpg"):
@@ -441,7 +468,7 @@ class TestImage:
         for ext in [".cur", ".icns", ".tif", ".tiff"]:
             assert ext in extensions
 
-    def test_effect_mandelbrot(self):
+    def test_effect_mandelbrot(self) -> None:
         # Arrange
         size = (512, 512)
         extent = (-3, -2.5, 2, 2.5)
@@ -454,7 +481,7 @@ class TestImage:
         assert im.size == (512, 512)
         assert_image_equal_tofile(im, "Tests/images/effect_mandelbrot.png")
 
-    def test_effect_mandelbrot_bad_arguments(self):
+    def test_effect_mandelbrot_bad_arguments(self) -> None:
         # Arrange
         size = (512, 512)
         # Get coordinates the wrong way round:
@@ -466,7 +493,7 @@ class TestImage:
         with pytest.raises(ValueError):
             Image.effect_mandelbrot(size, extent, quality)
 
-    def test_effect_noise(self):
+    def test_effect_noise(self) -> None:
         # Arrange
         size = (100, 100)
         sigma = 128
@@ -484,7 +511,7 @@ class TestImage:
         p4 = im.getpixel((0, 4))
         assert_not_all_same([p0, p1, p2, p3, p4])
 
-    def test_effect_spread(self):
+    def test_effect_spread(self) -> None:
         # Arrange
         im = hopper()
         distance = 10
@@ -496,7 +523,7 @@ class TestImage:
         assert im.size == (128, 128)
         assert_image_similar_tofile(im2, "Tests/images/effect_spread.png", 110)
 
-    def test_effect_spread_zero(self):
+    def test_effect_spread_zero(self) -> None:
         # Arrange
         im = hopper()
         distance = 0
@@ -507,12 +534,14 @@ class TestImage:
         # Assert
         assert_image_equal(im, im2)
 
-    def test_check_size(self):
+    def test_check_size(self) -> None:
         # Checking that the _check_size function throws value errors when we want it to
         with pytest.raises(ValueError):
-            Image.new("RGB", 0)  # not a tuple
+            # not a tuple
+            Image.new("RGB", 0)  # type: ignore[arg-type]
         with pytest.raises(ValueError):
-            Image.new("RGB", (0,))  # Tuple too short
+            # tuple too short
+            Image.new("RGB", (0,))  # type: ignore[arg-type]
         with pytest.raises(ValueError):
             Image.new("RGB", (-1, -1))  # w,h < 0
 
@@ -536,10 +565,10 @@ class TestImage:
         "PILLOW_VALGRIND_TEST" in os.environ, reason="Valgrind is slower"
     )
     @pytest.mark.parametrize("size", ((0, 100000000), (100000000, 0)))
-    def test_empty_image(self, size):
+    def test_empty_image(self, size: tuple[int, int]) -> None:
         Image.new("RGB", size)
 
-    def test_storage_neg(self):
+    def test_storage_neg(self) -> None:
         # Storage.c accepted negative values for xsize, ysize.  Was
         # test_neg_ppm, but the core function for that has been
         # removed Calling directly into core to test the error in
@@ -548,13 +577,14 @@ class TestImage:
         with pytest.raises(ValueError):
             Image.core.fill("RGB", (2, -2), (0, 0, 0))
 
-    def test_one_item_tuple(self):
+    def test_one_item_tuple(self) -> None:
         for mode in ("I", "F", "L"):
             im = Image.new(mode, (100, 100), (5,))
             px = im.load()
+            assert px is not None
             assert px[0, 0] == 5
 
-    def test_linear_gradient_wrong_mode(self):
+    def test_linear_gradient_wrong_mode(self) -> None:
         # Arrange
         wrong_mode = "RGB"
 
@@ -563,7 +593,7 @@ class TestImage:
             Image.linear_gradient(wrong_mode)
 
     @pytest.mark.parametrize("mode", ("L", "P", "I", "F"))
-    def test_linear_gradient(self, mode):
+    def test_linear_gradient(self, mode: str) -> None:
         # Arrange
         target_file = "Tests/images/linear_gradient.png"
 
@@ -579,7 +609,7 @@ class TestImage:
             target = target.convert(mode)
         assert_image_equal(im, target)
 
-    def test_radial_gradient_wrong_mode(self):
+    def test_radial_gradient_wrong_mode(self) -> None:
         # Arrange
         wrong_mode = "RGB"
 
@@ -588,7 +618,7 @@ class TestImage:
             Image.radial_gradient(wrong_mode)
 
     @pytest.mark.parametrize("mode", ("L", "P", "I", "F"))
-    def test_radial_gradient(self, mode):
+    def test_radial_gradient(self, mode: str) -> None:
         # Arrange
         target_file = "Tests/images/radial_gradient.png"
 
@@ -604,7 +634,7 @@ class TestImage:
             target = target.convert(mode)
         assert_image_equal(im, target)
 
-    def test_register_extensions(self):
+    def test_register_extensions(self) -> None:
         test_format = "a"
         exts = ["b", "c"]
         for ext in exts:
@@ -620,7 +650,7 @@ class TestImage:
 
         assert ext_individual == ext_multiple
 
-    def test_remap_palette(self):
+    def test_remap_palette(self) -> None:
         # Test identity transform
         with Image.open("Tests/images/hopper.gif") as im:
             assert_image_equal(im, im.remap_palette(list(range(256))))
@@ -639,14 +669,16 @@ class TestImage:
             with pytest.raises(ValueError):
                 im.remap_palette(None)
 
-    def test_remap_palette_transparency(self):
+    def test_remap_palette_transparency(self) -> None:
         im = Image.new("P", (1, 2), (0, 0, 0))
         im.putpixel((0, 1), (255, 0, 0))
         im.info["transparency"] = 0
 
         im_remapped = im.remap_palette([1, 0])
         assert im_remapped.info["transparency"] == 1
-        assert len(im_remapped.getpalette()) == 6
+        palette = im_remapped.getpalette()
+        assert palette is not None
+        assert len(palette) == 6
 
         # Test unused transparency
         im.info["transparency"] = 2
@@ -654,7 +686,7 @@ class TestImage:
         im_remapped = im.remap_palette([1, 0])
         assert "transparency" not in im_remapped.info
 
-    def test__new(self):
+    def test__new(self) -> None:
         im = hopper("RGB")
         im_p = hopper("P")
 
@@ -663,32 +695,40 @@ class TestImage:
         blank_p.palette = None
         blank_pa.palette = None
 
-        def _make_new(base_image, image, palette_result=None):
+        def _make_new(
+            base_image: Image.Image,
+            image: Image.Image,
+            palette_result: ImagePalette.ImagePalette | None = None,
+        ) -> None:
             new_image = base_image._new(image.im)
             assert new_image.mode == image.mode
             assert new_image.size == image.size
             assert new_image.info == base_image.info
             if palette_result is not None:
+                assert new_image.palette is not None
                 assert new_image.palette.tobytes() == palette_result.tobytes()
             else:
                 assert new_image.palette is None
 
-        _make_new(im, im_p, ImagePalette.ImagePalette(list(range(256)) * 3))
+        _make_new(im, im_p, ImagePalette.ImagePalette("RGB"))
         _make_new(im_p, im, None)
         _make_new(im, blank_p, ImagePalette.ImagePalette())
         _make_new(im, blank_pa, ImagePalette.ImagePalette())
 
-    def test_p_from_rgb_rgba(self):
-        for mode, color in [
+    @pytest.mark.parametrize(
+        "mode, color",
+        (
             ("RGB", "#DDEEFF"),
             ("RGB", (221, 238, 255)),
             ("RGBA", (221, 238, 255, 255)),
-        ]:
-            im = Image.new("P", (100, 100), color)
-            expected = Image.new(mode, (100, 100), color)
-            assert_image_equal(im.convert(mode), expected)
+        ),
+    )
+    def test_p_from_rgb_rgba(self, mode: str, color: str | tuple[int, ...]) -> None:
+        im = Image.new("P", (100, 100), color)
+        expected = Image.new(mode, (100, 100), color)
+        assert_image_equal(im.convert(mode), expected)
 
-    def test_no_resource_warning_on_save(self, tmp_path):
+    def test_no_resource_warning_on_save(self, tmp_path: Path) -> None:
         # https://github.com/python-pillow/Pillow/issues/835
         # Arrange
         test_file = "Tests/images/hopper.png"
@@ -697,9 +737,11 @@ class TestImage:
         # Act/Assert
         with Image.open(test_file) as im:
             with warnings.catch_warnings():
+                warnings.simplefilter("error")
+
                 im.save(temp_file)
 
-    def test_no_new_file_on_error(self, tmp_path):
+    def test_no_new_file_on_error(self, tmp_path: Path) -> None:
         temp_file = str(tmp_path / "temp.jpg")
 
         im = Image.new("RGB", (0, 0))
@@ -708,10 +750,10 @@ class TestImage:
 
         assert not os.path.exists(temp_file)
 
-    def test_load_on_nonexclusive_multiframe(self):
+    def test_load_on_nonexclusive_multiframe(self) -> None:
         with open("Tests/images/frozenpond.mpo", "rb") as fp:
 
-            def act(fp):
+            def act(fp: IO[bytes]) -> None:
                 im = Image.open(fp)
                 im.load()
 
@@ -722,7 +764,7 @@ class TestImage:
 
             assert not fp.closed
 
-    def test_empty_exif(self):
+    def test_empty_exif(self) -> None:
         with Image.open("Tests/images/exif.png") as im:
             exif = im.getexif()
         assert dict(exif)
@@ -735,10 +777,30 @@ class TestImage:
         exif.load(b"Exif\x00\x00")
         assert not dict(exif)
 
+    def test_duplicate_exif_header(self) -> None:
+        with Image.open("Tests/images/exif.png") as im:
+            im.load()
+            im.info["exif"] = b"Exif\x00\x00" + im.info["exif"]
+
+            exif = im.getexif()
+        assert exif[274] == 1
+
+    def test_empty_get_ifd(self) -> None:
+        exif = Image.Exif()
+        ifd = exif.get_ifd(0x8769)
+        assert ifd == {}
+
+        ifd[36864] = b"0220"
+        assert exif.get_ifd(0x8769) == {36864: b"0220"}
+
+        reloaded_exif = Image.Exif()
+        reloaded_exif.load(exif.tobytes())
+        assert reloaded_exif.get_ifd(0x8769) == {36864: b"0220"}
+
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_exif_jpeg(self, tmp_path):
+    def test_exif_jpeg(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/exif-72dpi-int.jpg") as im:  # Little endian
             exif = im.getexif()
             assert 258 not in exif
@@ -783,8 +845,7 @@ class TestImage:
             assert reloaded_exif[305] == "Pillow test"
 
     @skip_unless_feature("webp")
-    @skip_unless_feature("webp_anim")
-    def test_exif_webp(self, tmp_path):
+    def test_exif_webp(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/hopper.webp") as im:
             exif = im.getexif()
             assert exif == {}
@@ -794,7 +855,7 @@ class TestImage:
             exif[40963] = 455
             exif[305] = "Pillow test"
 
-            def check_exif():
+            def check_exif() -> None:
                 with Image.open(out) as reloaded:
                     reloaded_exif = reloaded.getexif()
                     assert reloaded_exif[258] == 8
@@ -806,7 +867,7 @@ class TestImage:
             im.save(out, exif=exif, save_all=True)
             check_exif()
 
-    def test_exif_png(self, tmp_path):
+    def test_exif_png(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/exif.png") as im:
             exif = im.getexif()
             assert exif == {274: 1}
@@ -822,7 +883,7 @@ class TestImage:
             reloaded_exif = reloaded.getexif()
             assert reloaded_exif == {258: 8, 40963: 455, 305: "Pillow test"}
 
-    def test_exif_interop(self):
+    def test_exif_interop(self) -> None:
         with Image.open("Tests/images/flower.jpg") as im:
             exif = im.getexif()
             assert exif.get_ifd(0xA005) == {
@@ -836,7 +897,7 @@ class TestImage:
             reloaded_exif.load(exif.tobytes())
             assert reloaded_exif.get_ifd(0xA005) == exif.get_ifd(0xA005)
 
-    def test_exif_ifd1(self):
+    def test_exif_ifd1(self) -> None:
         with Image.open("Tests/images/flower.jpg") as im:
             exif = im.getexif()
             assert exif.get_ifd(ExifTags.IFD.IFD1) == {
@@ -848,7 +909,7 @@ class TestImage:
                 283: 180.0,
             }
 
-    def test_exif_ifd(self):
+    def test_exif_ifd(self) -> None:
         with Image.open("Tests/images/flower.jpg") as im:
             exif = im.getexif()
         del exif.get_ifd(0x8769)[0xA005]
@@ -857,7 +918,7 @@ class TestImage:
         reloaded_exif.load(exif.tobytes())
         assert reloaded_exif.get_ifd(0x8769) == exif.get_ifd(0x8769)
 
-    def test_exif_load_from_fp(self):
+    def test_exif_load_from_fp(self) -> None:
         with Image.open("Tests/images/flower.jpg") as im:
             data = im.info["exif"]
             if data.startswith(b"Exif\x00\x00"):
@@ -878,7 +939,7 @@ class TestImage:
                 34665: 196,
             }
 
-    def test_exif_hide_offsets(self):
+    def test_exif_hide_offsets(self) -> None:
         with Image.open("Tests/images/flower.jpg") as im:
             exif = im.getexif()
 
@@ -903,19 +964,46 @@ class TestImage:
                     assert tag not in exif.get_ifd(0x8769)
                 assert exif.get_ifd(0xA005)
 
+    def test_empty_xmp(self) -> None:
+        with Image.open("Tests/images/hopper.gif") as im:
+            if ElementTree is None:
+                with pytest.warns(
+                    UserWarning,
+                    match="XMP data cannot be read without defusedxml dependency",
+                ):
+                    xmp = im.getxmp()
+            else:
+                xmp = im.getxmp()
+            assert xmp == {}
+
+    def test_getxmp_padded(self) -> None:
+        im = Image.new("RGB", (1, 1))
+        im.info["xmp"] = (
+            b'<?xpacket begin="\xef\xbb\xbf" id="W5M0MpCehiHzreSzNTczkc9d"?>\n'
+            b'<x:xmpmeta xmlns:x="adobe:ns:meta/" />\n<?xpacket end="w"?>\x00\x00'
+        )
+        if ElementTree is None:
+            with pytest.warns(
+                UserWarning,
+                match="XMP data cannot be read without defusedxml dependency",
+            ):
+                assert im.getxmp() == {}
+        else:
+            assert im.getxmp() == {"xmpmeta": None}
+
     @pytest.mark.parametrize("size", ((1, 0), (0, 1), (0, 0)))
-    def test_zero_tobytes(self, size):
+    def test_zero_tobytes(self, size: tuple[int, int]) -> None:
         im = Image.new("RGB", size)
         assert im.tobytes() == b""
 
     @pytest.mark.parametrize("size", ((1, 0), (0, 1), (0, 0)))
-    def test_zero_frombytes(self, size):
+    def test_zero_frombytes(self, size: tuple[int, int]) -> None:
         Image.frombytes("RGB", size, b"")
 
         im = Image.new("RGB", size)
         im.frombytes(b"")
 
-    def test_has_transparency_data(self):
+    def test_has_transparency_data(self) -> None:
         for mode in ("1", "L", "P", "RGB"):
             im = Image.new(mode, (1, 1))
             assert not im.has_transparency_data
@@ -937,12 +1025,14 @@ class TestImage:
         # P mode with RGBA palette
         im = Image.new("RGBA", (1, 1)).convert("P")
         assert im.mode == "P"
+        assert im.palette is not None
         assert im.palette.mode == "RGBA"
         assert im.has_transparency_data
 
-    def test_apply_transparency(self):
+    def test_apply_transparency(self) -> None:
         im = Image.new("P", (1, 1))
         im.putpalette((0, 0, 0, 1, 1, 1))
+        assert im.palette is not None
         assert im.palette.colors == {(0, 0, 0): 0, (1, 1, 1): 1}
 
         # Test that no transformation is applied without transparency
@@ -960,16 +1050,19 @@ class TestImage:
         im.putpalette((0, 0, 0, 255, 1, 1, 1, 128), "RGBA")
         im.info["transparency"] = 0
         im.apply_transparency()
+        assert im.palette is not None
         assert im.palette.colors == {(0, 0, 0, 0): 0, (1, 1, 1, 128): 1}
 
         # Test that transparency bytes are applied
         with Image.open("Tests/images/pil123p.png") as im:
             assert isinstance(im.info["transparency"], bytes)
+            assert im.palette is not None
             assert im.palette.colors[(27, 35, 6)] == 24
             im.apply_transparency()
+            assert im.palette is not None
             assert im.palette.colors[(27, 35, 6, 214)] == 24
 
-    def test_constants(self):
+    def test_constants(self) -> None:
         for enum in (
             Image.Transpose,
             Image.Transform,
@@ -994,34 +1087,29 @@ class TestImage:
             "01r_00.pcx",
         ],
     )
-    def test_overrun(self, path):
+    def test_overrun(self, path: str) -> None:
         """For overrun completeness, test as:
         valgrind pytest -qq Tests/test_image.py::TestImage::test_overrun | grep decode.c
         """
         with Image.open(os.path.join("Tests/images", path)) as im:
-            try:
+            with pytest.raises(OSError) as e:
                 im.load()
-                pytest.fail()
-            except OSError as e:
-                buffer_overrun = str(e) == "buffer overrun when reading image file"
-                truncated = "image file is truncated" in str(e)
+        buffer_overrun = str(e.value) == "buffer overrun when reading image file"
+        truncated = "image file is truncated" in str(e.value)
 
-                assert buffer_overrun or truncated
+        assert buffer_overrun or truncated
 
-    def test_fli_overrun2(self):
+    def test_fli_overrun2(self) -> None:
         with Image.open("Tests/images/fli_overrun2.bin") as im:
-            try:
+            with pytest.raises(OSError, match="buffer overrun when reading image file"):
                 im.seek(1)
-                pytest.fail()
-            except OSError as e:
-                assert str(e) == "buffer overrun when reading image file"
 
-    def test_exit_fp(self):
+    def test_exit_fp(self) -> None:
         with Image.new("L", (1, 1)) as im:
             pass
         assert not hasattr(im, "fp")
 
-    def test_close_graceful(self, caplog):
+    def test_close_graceful(self, caplog: pytest.LogCaptureFixture) -> None:
         with Image.open("Tests/images/hopper.jpg") as im:
             copy = im.copy()
             with caplog.at_level(logging.DEBUG):
@@ -1030,27 +1118,58 @@ class TestImage:
             assert len(caplog.records) == 0
             assert im.fp is None
 
+    def test_deprecation(self) -> None:
+        with pytest.warns(DeprecationWarning):
+            assert not Image.isImageType(None)
 
-class MockEncoder:
+
+class TestImageBytes:
+    @pytest.mark.parametrize("mode", Image.MODES + ["BGR;15", "BGR;16", "BGR;24"])
+    def test_roundtrip_bytes_constructor(self, mode: str) -> None:
+        im = hopper(mode)
+        source_bytes = im.tobytes()
+
+        if mode.startswith("BGR;"):
+            with pytest.warns(DeprecationWarning):
+                reloaded = Image.frombytes(mode, im.size, source_bytes)
+        else:
+            reloaded = Image.frombytes(mode, im.size, source_bytes)
+        assert reloaded.tobytes() == source_bytes
+
+    @pytest.mark.parametrize("mode", Image.MODES + ["BGR;15", "BGR;16", "BGR;24"])
+    def test_roundtrip_bytes_method(self, mode: str) -> None:
+        im = hopper(mode)
+        source_bytes = im.tobytes()
+
+        reloaded = helper_image_new(mode, im.size)
+        reloaded.frombytes(source_bytes)
+        assert reloaded.tobytes() == source_bytes
+
+    @pytest.mark.parametrize("mode", Image.MODES + ["BGR;15", "BGR;16", "BGR;24"])
+    def test_getdata_putdata(self, mode: str) -> None:
+        if is_big_endian() and mode == "BGR;15":
+            pytest.xfail("Known failure of BGR;15 on big-endian")
+        im = hopper(mode)
+        reloaded = helper_image_new(mode, im.size)
+        reloaded.putdata(im.getdata())
+        assert_image_equal(im, reloaded)
+
+
+class MockEncoder(ImageFile.PyEncoder):
     pass
 
 
-def mock_encode(*args):
-    encoder = MockEncoder()
-    encoder.args = args
-    return encoder
-
-
 class TestRegistry:
-    def test_encode_registry(self):
-        Image.register_encoder("MOCK", mock_encode)
+    def test_encode_registry(self) -> None:
+        Image.register_encoder("MOCK", MockEncoder)
         assert "MOCK" in Image.ENCODERS
 
         enc = Image._getencoder("RGB", "MOCK", ("args",), extra=("extra",))
 
         assert isinstance(enc, MockEncoder)
-        assert enc.args == ("RGB", "args", "extra")
+        assert enc.mode == "RGB"
+        assert enc.args == ("args", "extra")
 
-    def test_encode_registry_fail(self):
+    def test_encode_registry_fail(self) -> None:
         with pytest.raises(OSError):
             Image._getencoder("RGB", "DoesNotExist", ("args",), extra=("extra",))
