@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import pytest
 
 from PIL import Image
@@ -7,7 +8,7 @@ from .helper import assert_image_equal, hopper
 
 
 @pytest.mark.parametrize("mode", ("1", "P", "L", "RGB", "I", "F"))
-def test_crop(mode):
+def test_crop(mode: str) -> None:
     im = hopper(mode)
     assert_image_equal(im.crop(), im)
 
@@ -16,8 +17,8 @@ def test_crop(mode):
     assert cropped.size == (50, 50)
 
 
-def test_wide_crop():
-    def crop(*bbox):
+def test_wide_crop() -> None:
+    def crop(bbox: tuple[int, int, int, int]) -> tuple[int, ...]:
         i = im.crop(bbox)
         h = i.histogram()
         while h and not h[-1]:
@@ -26,34 +27,34 @@ def test_wide_crop():
 
     im = Image.new("L", (100, 100), 1)
 
-    assert crop(0, 0, 100, 100) == (0, 10000)
-    assert crop(25, 25, 75, 75) == (0, 2500)
+    assert crop((0, 0, 100, 100)) == (0, 10000)
+    assert crop((25, 25, 75, 75)) == (0, 2500)
 
     # sides
-    assert crop(-25, 0, 25, 50) == (1250, 1250)
-    assert crop(0, -25, 50, 25) == (1250, 1250)
-    assert crop(75, 0, 125, 50) == (1250, 1250)
-    assert crop(0, 75, 50, 125) == (1250, 1250)
+    assert crop((-25, 0, 25, 50)) == (1250, 1250)
+    assert crop((0, -25, 50, 25)) == (1250, 1250)
+    assert crop((75, 0, 125, 50)) == (1250, 1250)
+    assert crop((0, 75, 50, 125)) == (1250, 1250)
 
-    assert crop(-25, 25, 125, 75) == (2500, 5000)
-    assert crop(25, -25, 75, 125) == (2500, 5000)
+    assert crop((-25, 25, 125, 75)) == (2500, 5000)
+    assert crop((25, -25, 75, 125)) == (2500, 5000)
 
     # corners
-    assert crop(-25, -25, 25, 25) == (1875, 625)
-    assert crop(75, -25, 125, 25) == (1875, 625)
-    assert crop(75, 75, 125, 125) == (1875, 625)
-    assert crop(-25, 75, 25, 125) == (1875, 625)
+    assert crop((-25, -25, 25, 25)) == (1875, 625)
+    assert crop((75, -25, 125, 25)) == (1875, 625)
+    assert crop((75, 75, 125, 125)) == (1875, 625)
+    assert crop((-25, 75, 25, 125)) == (1875, 625)
 
 
 @pytest.mark.parametrize("box", ((8, 2, 2, 8), (2, 8, 8, 2), (8, 8, 2, 2)))
-def test_negative_crop(box):
+def test_negative_crop(box: tuple[int, int, int, int]) -> None:
     im = Image.new("RGB", (10, 10))
 
     with pytest.raises(ValueError):
         im.crop(box)
 
 
-def test_crop_float():
+def test_crop_float() -> None:
     # Check cropping floats are rounded to nearest integer
     # https://github.com/python-pillow/Pillow/issues/1744
 
@@ -68,7 +69,7 @@ def test_crop_float():
     assert cropped.size == (3, 5)
 
 
-def test_crop_crash():
+def test_crop_crash() -> None:
     # Image.crop crashes prepatch with an access violation
     # apparently a use after free on Windows, see
     # https://github.com/python-pillow/Pillow/issues/1077
@@ -86,7 +87,7 @@ def test_crop_crash():
     img.load()
 
 
-def test_crop_zero():
+def test_crop_zero() -> None:
     im = Image.new("RGB", (0, 0), "white")
 
     cropped = im.crop((0, 0, 0, 0))
