@@ -31,7 +31,9 @@ def test_libimagequant_quantize() -> None:
     converted = image.quantize(100, Image.Quantize.LIBIMAGEQUANT)
     assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 15)
-    assert len(converted.getcolors()) == 100
+    colors = converted.getcolors()
+    assert colors is not None
+    assert len(colors) == 100
 
 
 def test_octree_quantize() -> None:
@@ -39,7 +41,9 @@ def test_octree_quantize() -> None:
     converted = image.quantize(100, Image.Quantize.FASTOCTREE)
     assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 20)
-    assert len(converted.getcolors()) == 100
+    colors = converted.getcolors()
+    assert colors is not None
+    assert len(colors) == 100
 
 
 def test_rgba_quantize() -> None:
@@ -65,6 +69,7 @@ def test_quantize_no_dither() -> None:
 
     converted = image.quantize(dither=Image.Dither.NONE, palette=palette)
     assert converted.mode == "P"
+    assert converted.palette is not None
     assert converted.palette.palette == palette.palette.palette
 
 
@@ -77,9 +82,11 @@ def test_quantize_no_dither2() -> None:
     palette.putpalette(data)
     quantized = im.quantize(dither=Image.Dither.NONE, palette=palette)
 
+    assert quantized.palette is not None
     assert tuple(quantized.palette.palette) == data
 
     px = quantized.load()
+    assert px is not None
     for x in range(9):
         assert px[x, 0] == (0 if x < 5 else 1)
 
@@ -112,16 +119,19 @@ def test_colors() -> None:
     im = hopper()
     colors = 2
     converted = im.quantize(colors)
+    assert converted.palette is not None
     assert len(converted.palette.palette) == colors * len("RGB")
 
 
 def test_transparent_colors_equal() -> None:
     im = Image.new("RGBA", (1, 2), (0, 0, 0, 0))
     px = im.load()
+    assert px is not None
     px[0, 1] = (255, 255, 255, 0)
 
     converted = im.quantize()
     converted_px = converted.load()
+    assert converted_px is not None
     assert converted_px[0, 0] == converted_px[0, 1]
 
 
@@ -139,6 +149,8 @@ def test_palette(method: Image.Quantize, color: tuple[int, ...]) -> None:
 
     converted = im.quantize(method=method)
     converted_px = converted.load()
+    assert converted_px is not None
+    assert converted.palette is not None
     assert converted_px[0, 0] == converted.palette.colors[color]
 
 
@@ -154,4 +166,6 @@ def test_small_palette() -> None:
     im = im.quantize(palette=p)
 
     # Assert
-    assert len(im.getcolors()) == 2
+    quantized_colors = im.getcolors()
+    assert quantized_colors is not None
+    assert len(quantized_colors) == 2
