@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-import contextlib
 import os.path
-from typing import Sequence
+from collections.abc import Sequence
+from typing import Callable
 
 import pytest
 
@@ -857,6 +857,27 @@ def test_rounded_rectangle_corners(
     )
 
 
+def test_rounded_rectangle_joined_x_different_corners() -> None:
+    # Arrange
+    im = Image.new("RGB", (W, H))
+    draw = ImageDraw.Draw(im, "RGBA")
+
+    # Act
+    draw.rounded_rectangle(
+        (20, 10, 80, 90),
+        30,
+        fill="red",
+        outline="green",
+        width=5,
+        corners=(True, False, False, False),
+    )
+
+    # Assert
+    assert_image_equal_tofile(
+        im, "Tests/images/imagedraw_rounded_rectangle_joined_x_different_corners.png"
+    )
+
+
 @pytest.mark.parametrize(
     "xy, radius, type",
     [
@@ -1347,6 +1368,20 @@ def test_stroke() -> None:
         )
 
 
+@skip_unless_feature("freetype2")
+def test_stroke_float() -> None:
+    # Arrange
+    im = Image.new("RGB", (120, 130))
+    draw = ImageDraw.Draw(im)
+    font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 120)
+
+    # Act
+    draw.text((12, 12), "A", "#f00", font, stroke_width=0.5)
+
+    # Assert
+    assert_image_similar_tofile(im, "Tests/images/imagedraw_stroke_float.png", 3.1)
+
+
 @skip_unless_feature("freetype2")
 def test_stroke_descender() -> None:
     # Arrange
@@ -1422,25 +1457,44 @@ def test_default_font_size() -> None:
 
     im = Image.new("RGB", (220, 25))
     draw = ImageDraw.Draw(im)
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+
+    def check(func: Callable[[], None]) -> None:
+        if freetype_support:
+            func()
+        else:
+            with pytest.raises(ImportError):
+                func()
+
+    def draw_text() -> None:
         draw.text((0, 0), text, font_size=16)
         assert_image_equal_tofile(im, "Tests/images/imagedraw_default_font_size.png")
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_text)
+
+    def draw_textlength() -> None:
         assert draw.textlength(text, font_size=16) == 216
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_textlength)
+
+    def draw_textbbox() -> None:
         assert draw.textbbox((0, 0), text, font_size=16) == (0, 3, 216, 19)
 
+    check(draw_textbbox)
+
     im = Image.new("RGB", (220, 25))
     draw = ImageDraw.Draw(im)
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+
+    def draw_multiline_text() -> None:
         draw.multiline_text((0, 0), text, font_size=16)
         assert_image_equal_tofile(im, "Tests/images/imagedraw_default_font_size.png")
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_multiline_text)
+
+    def draw_multiline_textbbox() -> None:
         assert draw.multiline_textbbox((0, 0), text, font_size=16) == (0, 3, 216, 19)
 
+    check(draw_multiline_textbbox)
+
 
 @pytest.mark.parametrize("bbox", BBOX)
 def test_same_color_outline(bbox: Coords) -> None:
@@ -1620,6 +1674,9 @@ def test_continuous_horizontal_edges_polygon() -> None:
 def test_discontiguous_corners_polygon() -> None:
     img, draw = create_base_image_draw((84, 68))
     draw.polygon(((1, 21), (34, 4), (71, 1), (38, 18)), BLACK)
+    draw.polygon(
+        ((82, 29), (82, 26), (82, 24), (67, 22), (52, 29), (52, 15), (67, 22)), BLACK
+    )
     draw.polygon(((71, 44), (38, 27), (1, 24)), BLACK)
     draw.polygon(
         ((38, 66), (5, 49), (77, 49), (47, 66), (82, 63), (82, 47), (1, 47), (1, 63)),
