@@ -1,4 +1,7 @@
 from __future__ import annotations
+
+from pathlib import Path
+
 import pytest
 
 from PIL import Image, ImagePalette
@@ -6,19 +9,19 @@ from PIL import Image, ImagePalette
 from .helper import assert_image_equal, assert_image_equal_tofile
 
 
-def test_sanity():
+def test_sanity() -> None:
     palette = ImagePalette.ImagePalette("RGB", list(range(256)) * 3)
     assert len(palette.colors) == 256
 
 
-def test_reload():
+def test_reload() -> None:
     with Image.open("Tests/images/hopper.gif") as im:
         original = im.copy()
         im.palette.dirty = 1
         assert_image_equal(im.convert("RGB"), original.convert("RGB"))
 
 
-def test_getcolor():
+def test_getcolor() -> None:
     palette = ImagePalette.ImagePalette()
     assert len(palette.palette) == 0
     assert len(palette.colors) == 0
@@ -42,10 +45,10 @@ def test_getcolor():
 
     # Test unknown color specifier
     with pytest.raises(ValueError):
-        palette.getcolor("unknown")
+        palette.getcolor("unknown")  # type: ignore[arg-type]
 
 
-def test_getcolor_rgba_color_rgb_palette():
+def test_getcolor_rgba_color_rgb_palette() -> None:
     palette = ImagePalette.ImagePalette("RGB")
 
     # Opaque RGBA colors are converted
@@ -64,7 +67,7 @@ def test_getcolor_rgba_color_rgb_palette():
         (255, ImagePalette.ImagePalette("RGB", list(range(256)) * 3)),
     ],
 )
-def test_getcolor_not_special(index, palette):
+def test_getcolor_not_special(index: int, palette: ImagePalette.ImagePalette) -> None:
     im = Image.new("P", (1, 1))
 
     # Do not use transparency index as a new color
@@ -78,25 +81,25 @@ def test_getcolor_not_special(index, palette):
     assert index2 not in (index, index1)
 
 
-def test_file(tmp_path):
+def test_file(tmp_path: Path) -> None:
     palette = ImagePalette.ImagePalette("RGB", list(range(256)) * 3)
 
     f = str(tmp_path / "temp.lut")
 
     palette.save(f)
 
-    p = ImagePalette.load(f)
+    lut = ImagePalette.load(f)
 
     # load returns raw palette information
-    assert len(p[0]) == 768
-    assert p[1] == "RGB"
+    assert len(lut[0]) == 768
+    assert lut[1] == "RGB"
 
-    p = ImagePalette.raw(p[1], p[0])
+    p = ImagePalette.raw(lut[1], lut[0])
     assert isinstance(p, ImagePalette.ImagePalette)
     assert p.palette == palette.tobytes()
 
 
-def test_make_linear_lut():
+def test_make_linear_lut() -> None:
     # Arrange
     black = 0
     white = 255
@@ -112,7 +115,7 @@ def test_make_linear_lut():
         assert lut[i] == i
 
 
-def test_make_linear_lut_not_yet_implemented():
+def test_make_linear_lut_not_yet_implemented() -> None:
     # Update after FIXME
     # Arrange
     black = 1
@@ -123,7 +126,7 @@ def test_make_linear_lut_not_yet_implemented():
         ImagePalette.make_linear_lut(black, white)
 
 
-def test_make_gamma_lut():
+def test_make_gamma_lut() -> None:
     # Arrange
     exp = 5
 
@@ -141,7 +144,7 @@ def test_make_gamma_lut():
     assert lut[255] == 255
 
 
-def test_rawmode_valueerrors(tmp_path):
+def test_rawmode_valueerrors(tmp_path: Path) -> None:
     # Arrange
     palette = ImagePalette.raw("RGB", list(range(256)) * 3)
 
@@ -155,7 +158,7 @@ def test_rawmode_valueerrors(tmp_path):
         palette.save(f)
 
 
-def test_getdata():
+def test_getdata() -> None:
     # Arrange
     data_in = list(range(256)) * 3
     palette = ImagePalette.ImagePalette("RGB", data_in)
@@ -167,7 +170,7 @@ def test_getdata():
     assert mode == "RGB"
 
 
-def test_rawmode_getdata():
+def test_rawmode_getdata() -> None:
     # Arrange
     data_in = list(range(256)) * 3
     palette = ImagePalette.raw("RGB", data_in)
@@ -180,7 +183,7 @@ def test_rawmode_getdata():
     assert data_in == data_out
 
 
-def test_2bit_palette(tmp_path):
+def test_2bit_palette(tmp_path: Path) -> None:
     # issue #2258, 2 bit palettes are corrupted.
     outfile = str(tmp_path / "temp.png")
 
@@ -192,6 +195,6 @@ def test_2bit_palette(tmp_path):
     assert_image_equal_tofile(img, outfile)
 
 
-def test_invalid_palette():
+def test_invalid_palette() -> None:
     with pytest.raises(OSError):
         ImagePalette.load("Tests/images/hopper.jpg")
