@@ -1,4 +1,7 @@
 from __future__ import annotations
+
+from pathlib import Path
+
 import pytest
 
 from PIL import Image, ImageSequence, TiffImagePlugin
@@ -6,7 +9,7 @@ from PIL import Image, ImageSequence, TiffImagePlugin
 from .helper import assert_image_equal, hopper, skip_unless_feature
 
 
-def test_sanity(tmp_path):
+def test_sanity(tmp_path: Path) -> None:
     test_file = str(tmp_path / "temp.im")
 
     im = hopper("RGB")
@@ -23,10 +26,10 @@ def test_sanity(tmp_path):
     assert index == 1
 
     with pytest.raises(AttributeError):
-        ImageSequence.Iterator(0)
+        ImageSequence.Iterator(0)  # type: ignore[arg-type]
 
 
-def test_iterator():
+def test_iterator() -> None:
     with Image.open("Tests/images/multipage.tiff") as im:
         i = ImageSequence.Iterator(im)
         for index in range(0, im.n_frames):
@@ -37,14 +40,18 @@ def test_iterator():
             next(i)
 
 
-def test_iterator_min_frame():
+def test_iterator_min_frame() -> None:
     with Image.open("Tests/images/hopper.psd") as im:
         i = ImageSequence.Iterator(im)
         for index in range(1, im.n_frames):
             assert i[index] == next(i)
 
 
-def _test_multipage_tiff():
+@pytest.mark.parametrize(
+    "libtiff", (pytest.param(True, marks=skip_unless_feature("libtiff")), False)
+)
+def test_multipage_tiff(monkeypatch: pytest.MonkeyPatch, libtiff: bool) -> None:
+    monkeypatch.setattr(TiffImagePlugin, "READ_LIBTIFF", libtiff)
     with Image.open("Tests/images/multipage.tiff") as im:
         for index, frame in enumerate(ImageSequence.Iterator(im)):
             frame.load()
@@ -52,29 +59,19 @@ def _test_multipage_tiff():
             frame.convert("RGB")
 
 
-def test_tiff():
-    _test_multipage_tiff()
-
-
-@skip_unless_feature("libtiff")
-def test_libtiff():
-    TiffImagePlugin.READ_LIBTIFF = True
-    _test_multipage_tiff()
-    TiffImagePlugin.READ_LIBTIFF = False
-
-
-def test_consecutive():
+def test_consecutive() -> None:
     with Image.open("Tests/images/multipage.tiff") as im:
         first_frame = None
         for frame in ImageSequence.Iterator(im):
             if first_frame is None:
                 first_frame = frame.copy()
+        assert first_frame is not None
         for frame in ImageSequence.Iterator(im):
             assert_image_equal(frame, first_frame)
             break
 
 
-def test_palette_mmap():
+def test_palette_mmap() -> None:
     # Using mmap in ImageFile can require to reload the palette.
     with Image.open("Tests/images/multipage-mmap.tiff") as im:
         color1 = im.getpalette()[:3]
@@ -83,7 +80,7 @@ def test_palette_mmap():
         assert color1 == color2
 
 
-def test_all_frames():
+def test_all_frames() -> None:
     # Test a single image
     with Image.open("Tests/images/iss634.gif") as im:
         ims = ImageSequence.all_frames(im)
