@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import sys
 
 import pytest
@@ -6,7 +7,7 @@ import pytest
 from PIL import Image
 
 
-def test_overflow():
+def test_overflow() -> None:
     # There is the potential to overflow comparisons in map.c
     # if there are > SIZE_MAX bytes in the image or if
     # the file encodes an offset that makes
@@ -24,7 +25,7 @@ def test_overflow():
     Image.MAX_IMAGE_PIXELS = max_pixels
 
 
-def test_tobytes():
+def test_tobytes() -> None:
     # Note that this image triggers the decompression bomb warning:
     max_pixels = Image.MAX_IMAGE_PIXELS
     Image.MAX_IMAGE_PIXELS = None
@@ -38,7 +39,7 @@ def test_tobytes():
 
 
 @pytest.mark.skipif(sys.maxsize <= 2**32, reason="Requires 64-bit system")
-def test_ysize():
+def test_ysize() -> None:
     numpy = pytest.importorskip("numpy", reason="NumPy not installed")
 
     # Should not raise 'Integer overflow in ysize'
