@@ -1,14 +1,14 @@
 from __future__ import annotations
+
 import os
 import sys
 from io import BytesIO
-
-import pytest
+from pathlib import Path
 
 from PIL import Image, PSDraw
 
 
-def _create_document(ps):
+def _create_document(ps: PSDraw.PSDraw) -> None:
     title = "hopper"
     box = (1 * 72, 2 * 72, 7 * 72, 10 * 72)  # in points
 
@@ -30,7 +30,7 @@ def _create_document(ps):
     ps.end_document()
 
 
-def test_draw_postscript(tmp_path):
+def test_draw_postscript(tmp_path: Path) -> None:
     # Based on Pillow tutorial, but there is no textsize:
     # https://pillow.readthedocs.io/en/latest/handbook/tutorial.html#drawing-postscript
 
@@ -47,19 +47,14 @@ def test_draw_postscript(tmp_path):
     assert os.path.getsize(tempfile) > 0
 
 
-@pytest.mark.parametrize("buffer", (True, False))
-def test_stdout(buffer):
+def test_stdout() -> None:
     # Temporarily redirect stdout
     old_stdout = sys.stdout
 
-    if buffer:
+    class MyStdOut:
+        buffer = BytesIO()
 
-        class MyStdOut:
-            buffer = BytesIO()
-
-        mystdout = MyStdOut()
-    else:
-        mystdout = BytesIO()
+    mystdout = MyStdOut()
 
     sys.stdout = mystdout
 
@@ -69,6 +64,4 @@ def test_stdout(buffer):
     # Reset stdout
     sys.stdout = old_stdout
 
-    if buffer:
-        mystdout = mystdout.buffer
-    assert mystdout.getvalue() != b""
+    assert mystdout.buffer.getvalue() != b""
