@@ -14,17 +14,25 @@
 #
 # See the README file for information on usage and redistribution.
 #
+from __future__ import annotations
 
+import os
 
 from . import Image, ImageFile, ImagePalette
-from ._binary import i8, i16le as i16, i32le as i32, o8
+from ._binary import i16le as i16
+from ._binary import i32le as i32
+from ._binary import o8
 
 #
 # decoder
 
 
-def _accept(prefix):
-    return len(prefix) >= 6 and i16(prefix[4:6]) in [0xAF11, 0xAF12]
+def _accept(prefix: bytes) -> bool:
+    return (
+        len(prefix) >= 6
+        and i16(prefix, 4) in [0xAF11, 0xAF12]
+        and i16(prefix, 14) in [0, 3]  # flags
+    )
 
 
 ##
@@ -33,32 +41,28 @@ def _accept(prefix):
 
 
 class FliImageFile(ImageFile.ImageFile):
-
     format = "FLI"
     format_description = "Autodesk FLI/FLC Animation"
     _close_exclusive_fp_after_loading = False
 
-    def _open(self):
-
+    def _open(self) -> None:
         # HEAD
         s = self.fp.read(128)
-        magic = i16(s[4:6])
-        if not (
-            magic in [0xAF11, 0xAF12]
-            and i16(s[14:16]) in [0, 3]  # flags
-            and s[20:22] == b"\x00\x00"  # reserved
-        ):
-            raise SyntaxError("not an FLI/FLC file")
+        if not (_accept(s) and s[20:22] == b"\x00\x00"):
+            msg = "not an FLI/FLC file"
+            raise SyntaxError(msg)
 
         # frames
-        self.__framecount = i16(s[6:8])
+        self.n_frames = i16(s, 6)
+        self.is_animated = self.n_frames > 1
 
         # image characteristics
-        self.mode = "P"
-        self._size = i16(s[8:10]), i16(s[10:12])
+        self._mode = "P"
+        self._size = i16(s, 8), i16(s, 10)
 
         # animation speed
-        duration = i32(s[16:20])
+        duration = i32(s, 16)
+        magic = i16(s, 4)
         if magic == 0xAF11:
             duration = (duration * 1000) // 70
         self.info["duration"] = duration
@@ -70,55 +74,57 @@ class FliImageFile(ImageFile.ImageFile):
 
         self.__offset = 128
 
-        if i16(s[4:6]) == 0xF100:
+        if i16(s, 4) == 0xF100:
             # prefix chunk; ignore it
             self.__offset = self.__offset + i32(s)
+            self.fp.seek(self.__offset)
             s = self.fp.read(16)
 
-        if i16(s[4:6]) == 0xF1FA:
+        if i16(s, 4) == 0xF1FA:
             # look for palette chunk
-            s = self.fp.read(6)
-            if i16(s[4:6]) == 11:
-                self._palette(palette, 2)
-            elif i16(s[4:6]) == 4:
-                self._palette(palette, 0)
+            number_of_subchunks = i16(s, 6)
+            chunk_size: int | None = None
+            for _ in range(number_of_subchunks):
+                if chunk_size is not None:
+                    self.fp.seek(chunk_size - 6, os.SEEK_CUR)
+                s = self.fp.read(6)
+                chunk_type = i16(s, 4)
+                if chunk_type in (4, 11):
+                    self._palette(palette, 2 if chunk_type == 11 else 0)
+                    break
+                chunk_size = i32(s)
+                if not chunk_size:
+                    break
 
-        palette = [o8(r) + o8(g) + o8(b) for (r, g, b) in palette]
-        self.palette = ImagePalette.raw("RGB", b"".join(palette))
+        self.palette = ImagePalette.raw(
+            "RGB", b"".join(o8(r) + o8(g) + o8(b) for (r, g, b) in palette)
+        )
 
         # set things up to decode first frame
         self.__frame = -1
-        self.__fp = self.fp
+        self._fp = self.fp
         self.__rewind = self.fp.tell()
         self.seek(0)
 
-    def _palette(self, palette, shift):
+    def _palette(self, palette: list[tuple[int, int, int]], shift: int) -> None:
         # load palette
 
         i = 0
         for e in range(i16(self.fp.read(2))):
             s = self.fp.read(2)
-            i = i + i8(s[0])
-            n = i8(s[1])
+            i = i + s[0]
+            n = s[1]
             if n == 0:
                 n = 256
             s = self.fp.read(n * 3)
             for n in range(0, len(s), 3):
-                r = i8(s[n]) << shift
-                g = i8(s[n + 1]) << shift
-                b = i8(s[n + 2]) << shift
+                r = s[n] << shift
+                g = s[n + 1] << shift
+                b = s[n + 2] << shift
                 palette[i] = (r, g, b)
                 i += 1
 
-    @property
-    def n_frames(self):
-        return self.__framecount
-
-    @property
-    def is_animated(self):
-        return self.__framecount > 1
-
-    def seek(self, frame):
+    def seek(self, frame: int) -> None:
         if not self._seek_check(frame):
             return
         if frame < self.__frame:
@@ -127,46 +133,39 @@ class FliImageFile(ImageFile.ImageFile):
         for f in range(self.__frame + 1, frame + 1):
             self._seek(f)
 
-    def _seek(self, frame):
+    def _seek(self, frame: int) -> None:
         if frame == 0:
             self.__frame = -1
-            self.__fp.seek(self.__rewind)
+            self._fp.seek(self.__rewind)
             self.__offset = 128
         else:
             # ensure that the previous frame was loaded
             self.load()
 
         if frame != self.__frame + 1:
-            raise ValueError("cannot seek to frame %d" % frame)
+            msg = f"cannot seek to frame {frame}"
+            raise ValueError(msg)
         self.__frame = frame
 
         # move to next frame
-        self.fp = self.__fp
+        self.fp = self._fp
         self.fp.seek(self.__offset)
 
         s = self.fp.read(4)
         if not s:
-            raise EOFError
+            msg = "missing frame size"
+            raise EOFError(msg)
 
         framesize = i32(s)
 
         self.decodermaxblock = framesize
-        self.tile = [("fli", (0, 0) + self.size, self.__offset, None)]
+        self.tile = [ImageFile._Tile("fli", (0, 0) + self.size, self.__offset)]
 
         self.__offset += framesize
 
-    def tell(self):
+    def tell(self) -> int:
         return self.__frame
 
-    def _close__fp(self):
-        try:
-            if self.__fp != self.fp:
-                self.__fp.close()
-        except AttributeError:
-            pass
-        finally:
-            self.__fp = None
-
 
 #
 # registry
