@@ -14,15 +14,17 @@
 #
 # See the README file for information on usage and redistribution.
 #
+from __future__ import annotations
+
 import olefile
 
 from . import Image, ImageFile
-from ._binary import i8, i32le as i32
+from ._binary import i32le as i32
 
 # we map from colour field tuples to (mode, rawmode) descriptors
 MODES = {
     # opacity
-    (0x00007FFE): ("A", "L"),
+    (0x00007FFE,): ("A", "L"),
     # monochrome
     (0x00010000,): ("L", "L"),
     (0x00018000, 0x00017FFE): ("RGBA", "LA"),
@@ -39,7 +41,7 @@ MODES = {
 # --------------------------------------------------------------------
 
 
-def _accept(prefix):
+def _accept(prefix: bytes) -> bool:
     return prefix[:8] == olefile.MAGIC
 
 
@@ -48,41 +50,45 @@ def _accept(prefix):
 
 
 class FpxImageFile(ImageFile.ImageFile):
-
     format = "FPX"
     format_description = "FlashPix"
 
-    def _open(self):
+    def _open(self) -> None:
         #
         # read the OLE directory and see if this is a likely
         # to be a FlashPix file
 
         try:
             self.ole = olefile.OleFileIO(self.fp)
-        except OSError:
-            raise SyntaxError("not an FPX file; invalid OLE file")
+        except OSError as e:
+            msg = "not an FPX file; invalid OLE file"
+            raise SyntaxError(msg) from e
 
-        if self.ole.root.clsid != "56616700-C154-11CE-8553-00AA00A1F95B":
-            raise SyntaxError("not an FPX file; bad root CLSID")
+        root = self.ole.root
+        if not root or root.clsid != "56616700-C154-11CE-8553-00AA00A1F95B":
+            msg = "not an FPX file; bad root CLSID"
+            raise SyntaxError(msg)
 
         self._open_index(1)
 
-    def _open_index(self, index=1):
+    def _open_index(self, index: int = 1) -> None:
         #
         # get the Image Contents Property Set
 
         prop = self.ole.getproperties(
-            ["Data Object Store %06d" % index, "\005Image Contents"]
+            [f"Data Object Store {index:06d}", "\005Image Contents"]
         )
 
         # size (highest resolution)
 
+        assert isinstance(prop[0x1000002], int)
+        assert isinstance(prop[0x1000003], int)
         self._size = prop[0x1000002], prop[0x1000003]
 
         size = max(self.size)
         i = 1
         while size > 64:
-            size = size / 2
+            size = size // 2
             i += 1
         self.maxid = i - 1
 
@@ -96,15 +102,14 @@ class FpxImageFile(ImageFile.ImageFile):
 
         s = prop[0x2000002 | id]
 
-        colors = []
-        bands = i32(s, 4)
-        if bands > 4:
-            raise IOError("Invalid number of bands")
-        for i in range(bands):
-            # note: for now, we ignore the "uncalibrated" flag
-            colors.append(i32(s, 8 + i * 4) & 0x7FFFFFFF)
+        if not isinstance(s, bytes) or (bands := i32(s, 4)) > 4:
+            msg = "Invalid number of bands"
+            raise OSError(msg)
 
-        self.mode, self.rawmode = MODES[tuple(colors)]
+        # note: for now, we ignore the "uncalibrated" flag
+        colors = tuple(i32(s, 8 + i * 4) & 0x7FFFFFFF for i in range(bands))
+
+        self._mode, self.rawmode = MODES[colors]
 
         # load JPEG tables, if any
         self.jpeg = {}
@@ -115,13 +120,13 @@ class FpxImageFile(ImageFile.ImageFile):
 
         self._open_subimage(1, self.maxid)
 
-    def _open_subimage(self, index=1, subimage=0):
+    def _open_subimage(self, index: int = 1, subimage: int = 0) -> None:
         #
         # setup tile descriptors for a given subimage
 
         stream = [
-            "Data Object Store %06d" % index,
-            "Resolution %04d" % subimage,
+            f"Data Object Store {index:06d}",
+            f"Resolution {subimage:04d}",
             "Subimage 0000 Header",
         ]
 
@@ -141,7 +146,8 @@ class FpxImageFile(ImageFile.ImageFile):
         length = i32(s, 32)
 
         if size != self.size:
-            raise OSError("subimage mismatch")
+            msg = "subimage mismatch"
+            raise OSError(msg)
 
         # get tile descriptors
         fp.seek(28 + offset)
@@ -153,35 +159,35 @@ class FpxImageFile(ImageFile.ImageFile):
         self.tile = []
 
         for i in range(0, len(s), length):
+            x1 = min(xsize, x + xtile)
+            y1 = min(ysize, y + ytile)
 
             compression = i32(s, i + 8)
 
             if compression == 0:
                 self.tile.append(
-                    (
+                    ImageFile._Tile(
                         "raw",
-                        (x, y, x + xtile, y + ytile),
+                        (x, y, x1, y1),
                         i32(s, i) + 28,
-                        (self.rawmode),
+                        self.rawmode,
                     )
                 )
 
             elif compression == 1:
-
                 # FIXME: the fill decoder is not implemented
                 self.tile.append(
-                    (
+                    ImageFile._Tile(
                         "fill",
-                        (x, y, x + xtile, y + ytile),
+                        (x, y, x1, y1),
                         i32(s, i) + 28,
                         (self.rawmode, s[12:16]),
                     )
                 )
 
             elif compression == 2:
-
-                internal_color_conversion = i8(s[14])
-                jpeg_tables = i8(s[15])
+                internal_color_conversion = s[14]
+                jpeg_tables = s[15]
                 rawmode = self.rawmode
 
                 if internal_color_conversion:
@@ -199,9 +205,9 @@ class FpxImageFile(ImageFile.ImageFile):
                     jpegmode = rawmode
 
                 self.tile.append(
-                    (
+                    ImageFile._Tile(
                         "jpeg",
-                        (x, y, x + xtile, y + ytile),
+                        (x, y, x1, y1),
                         i32(s, i) + 28,
                         (rawmode, jpegmode),
                     )
@@ -214,7 +220,8 @@ class FpxImageFile(ImageFile.ImageFile):
                     self.tile_prefix = self.jpeg[jpeg_tables]
 
             else:
-                raise OSError("unknown/invalid compression")
+                msg = "unknown/invalid compression"
+                raise OSError(msg)
 
             x = x + xtile
             if x >= xsize:
@@ -223,15 +230,23 @@ class FpxImageFile(ImageFile.ImageFile):
                     break  # isn't really required
 
         self.stream = stream
+        self._fp = self.fp
         self.fp = None
 
-    def load(self):
-
+    def load(self) -> Image.core.PixelAccess | None:
         if not self.fp:
             self.fp = self.ole.openstream(self.stream[:2] + ["Subimage 0000 Data"])
 
         return ImageFile.ImageFile.load(self)
 
+    def close(self) -> None:
+        self.ole.close()
+        super().close()
+
+    def __exit__(self, *args: object) -> None:
+        self.ole.close()
+        super().__exit__()
+
 
 #
 # --------------------------------------------------------------------
