@@ -67,7 +67,8 @@ j2k_skip(OPJ_OFF_T p_nb_bytes, void *p_user_data) {
 /* -------------------------------------------------------------------- */
 
 typedef void (*j2k_unpacker_t)(
-    opj_image_t *in, const JPEG2KTILEINFO *tileInfo, const UINT8 *data, Imaging im);
+    opj_image_t *in, const JPEG2KTILEINFO *tileInfo, const UINT8 *data, Imaging im
+);
 
 struct j2k_decode_unpacker {
     const char *mode;
@@ -89,10 +90,8 @@ j2ku_shift(unsigned x, int n) {
 
 static void
 j2ku_gray_l(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -145,10 +144,8 @@ j2ku_gray_l(
 
 static void
 j2ku_gray_i(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -204,10 +201,8 @@ j2ku_gray_i(
 
 static void
 j2ku_gray_rgb(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -268,10 +263,8 @@ j2ku_gray_rgb(
 
 static void
 j2ku_graya_la(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -347,10 +340,8 @@ j2ku_graya_la(
 
 static void
 j2ku_srgb_rgb(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -413,10 +404,8 @@ j2ku_srgb_rgb(
 
 static void
 j2ku_sycc_rgb(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -482,10 +471,8 @@ j2ku_sycc_rgb(
 
 static void
 j2ku_srgba_rgba(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -547,10 +534,8 @@ j2ku_srgba_rgba(
 
 static void
 j2ku_sycca_rgba(
-    opj_image_t *in,
-    const JPEG2KTILEINFO *tileinfo,
-    const UINT8 *tiledata,
-    Imaging im) {
+    opj_image_t *in, const JPEG2KTILEINFO *tileinfo, const UINT8 *tiledata, Imaging im
+) {
     unsigned x0 = tileinfo->x0 - in->x0, y0 = tileinfo->y0 - in->y0;
     unsigned w = tileinfo->x1 - tileinfo->x0;
     unsigned h = tileinfo->y1 - tileinfo->y0;
@@ -655,7 +640,7 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
     opj_dparameters_t params;
     OPJ_COLOR_SPACE color_space;
     j2k_unpacker_t unpack = NULL;
-    size_t buffer_size = 0, tile_bytes = 0;
+    size_t tile_bytes = 0;
     unsigned n, tile_height, tile_width;
     int subsampling;
     int total_component_width = 0;
@@ -713,8 +698,7 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
     }
 
     /* Check that this image is something we can handle */
-    if (image->numcomps < 1 || image->numcomps > 4 ||
-        image->color_space == OPJ_CLRSPC_UNKNOWN) {
+    if (image->numcomps < 1 || image->numcomps > 4) {
         state->errcode = IMAGING_CODEC_BROKEN;
         state->state = J2K_STATE_FAILED;
         goto quick_exit;
@@ -759,7 +743,7 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
     /* Find the correct unpacker */
     color_space = image->color_space;
 
-    if (color_space == OPJ_CLRSPC_UNSPECIFIED) {
+    if (color_space == OPJ_CLRSPC_UNKNOWN || color_space == OPJ_CLRSPC_UNSPECIFIED) {
         switch (image->numcomps) {
             case 1:
             case 2:
@@ -815,7 +799,8 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
                 &tile_info.x1,
                 &tile_info.y1,
                 &tile_info.nb_comps,
-                &should_continue)) {
+                &should_continue
+            )) {
             state->errcode = IMAGING_CODEC_BROKEN;
             state->state = J2K_STATE_FAILED;
             goto quick_exit;
@@ -885,7 +870,7 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
             tile_info.data_size = tile_bytes;
         }
 
-        if (buffer_size < tile_info.data_size) {
+        if (tile_info.data_size > 0) {
             /* malloc check ok, overflow and tile size sanity check above */
             UINT8 *new = realloc(state->buffer, tile_info.data_size);
             if (!new) {
@@ -898,7 +883,6 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
                to valgrind errors. */
             memset(new, 0, tile_info.data_size);
             state->buffer = new;
-            buffer_size = tile_info.data_size;
         }
 
         if (!opj_decode_tile_data(
@@ -906,7 +890,8 @@ j2k_decode_entry(Imaging im, ImagingCodecState state) {
                 tile_info.tile_index,
                 (OPJ_BYTE *)state->buffer,
                 tile_info.data_size,
-                stream)) {
+                stream
+            )) {
             state->errcode = IMAGING_CODEC_BROKEN;
             state->state = J2K_STATE_FAILED;
             goto quick_exit;
@@ -993,10 +978,3 @@ ImagingJpeg2KVersion(void) {
 }
 
 #endif /* HAVE_OPENJPEG */
-
-/*
- * Local Variables:
- * c-basic-offset: 4
- * End:
- *
- */
