@@ -175,7 +175,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                         i,
                         &context->qtables[i * DCTSIZE2],
                         quality,
-                        FALSE);
+                        FALSE
+                    );
                     context->cinfo.comp_info[i].quant_tbl_no = i;
                     last_q = i;
                 }
@@ -183,7 +184,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                     // jpeg_set_defaults created two qtables internally, but we only
                     // wanted one.
                     jpeg_add_quant_table(
-                        &context->cinfo, 1, &context->qtables[0], quality, FALSE);
+                        &context->cinfo, 1, &context->qtables[0], quality, FALSE
+                    );
                 }
                 for (i = last_q; i < context->cinfo.num_components; i++) {
                     context->cinfo.comp_info[i].quant_tbl_no = last_q;
@@ -273,7 +275,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                     &context->cinfo,
                     JPEG_APP0 + 1,
                     (unsigned char *)context->rawExif,
-                    context->rawExifLen);
+                    context->rawExifLen
+                );
             }
 
             state->state++;
@@ -289,7 +292,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                 memcpy(
                     context->destination.pub.next_output_byte,
                     context->extra + context->extra_offset,
-                    n);
+                    n
+                );
                 context->destination.pub.next_output_byte += n;
                 context->destination.pub.free_in_buffer -= n;
                 context->extra_offset += n;
@@ -309,7 +313,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                     &context->cinfo,
                     JPEG_COM,
                     (unsigned char *)context->comment,
-                    context->comment_size);
+                    context->comment_size
+                );
             }
             state->state++;
 
@@ -324,7 +329,8 @@ ImagingJpegEncode(Imaging im, ImagingCodecState state, UINT8 *buf, int bytes) {
                     state->buffer,
                     (UINT8 *)im->image[state->y + state->yoff] +
                         state->xoff * im->pixelsize,
-                    state->xsize);
+                    state->xsize
+                );
                 ok = jpeg_write_scanlines(&context->cinfo, &state->buffer, 1);
                 if (ok != 1) {
                     break;
