@@ -113,7 +113,9 @@ expandrow(UINT8 *dest, UINT8 *src, int n, int z, int xsize, UINT8 *end_of_buffer
 }
 
 static int
-expandrow2(UINT8 *dest, const UINT8 *src, int n, int z, int xsize, UINT8 *end_of_buffer) {
+expandrow2(
+    UINT8 *dest, const UINT8 *src, int n, int z, int xsize, UINT8 *end_of_buffer
+) {
     UINT8 pixel, count;
     int x = 0;
 
@@ -181,7 +183,7 @@ ImagingSgiRleDecode(Imaging im, ImagingCodecState state, UINT8 *buf, Py_ssize_t
        each with 4 bytes per element of tablen
        Check here before we allocate any memory
     */
-    if (c->bufsize < 8 * c->tablen) {
+    if (c->bufsize < 8 * (int64_t)c->tablen) {
         state->errcode = IMAGING_CODEC_OVERRUN;
         return -1;
     }
@@ -193,11 +195,11 @@ ImagingSgiRleDecode(Imaging im, ImagingCodecState state, UINT8 *buf, Py_ssize_t
     }
     _imaging_seek_pyFd(state->fd, SGI_HEADER_SIZE, SEEK_SET);
     if (_imaging_read_pyFd(state->fd, (char *)ptr, c->bufsize) != c->bufsize) {
+        free(ptr);
         state->errcode = IMAGING_CODEC_UNKNOWN;
         return -1;
     }
 
-
     /* decoder initialization */
     state->count = 0;
     state->y = 0;
@@ -252,7 +254,8 @@ ImagingSgiRleDecode(Imaging im, ImagingCodecState state, UINT8 *buf, Py_ssize_t
                     c->rlelength,
                     im->bands,
                     im->xsize,
-                    &ptr[c->bufsize-1]);
+                    &ptr[c->bufsize - 1]
+                );
             } else {
                 status = expandrow2(
                     &state->buffer[c->channo * 2],
@@ -260,7 +263,8 @@ ImagingSgiRleDecode(Imaging im, ImagingCodecState state, UINT8 *buf, Py_ssize_t
                     c->rlelength,
                     im->bands,
                     im->xsize,
-                    &ptr[c->bufsize-1]);
+                    &ptr[c->bufsize - 1]
+                );
             }
             if (status == -1) {
                 state->errcode = IMAGING_CODEC_OVERRUN;
@@ -268,7 +272,6 @@ ImagingSgiRleDecode(Imaging im, ImagingCodecState state, UINT8 *buf, Py_ssize_t
             } else if (status == 1) {
                 goto sgi_finish_decode;
             }
-
         }
 
         /* store decompressed data in image */
