@@ -104,7 +104,8 @@ static UINT8 BITFLIP[] = {
     3,  131, 67, 195, 35, 163, 99,  227, 19, 147, 83, 211, 51, 179, 115, 243,
     11, 139, 75, 203, 43, 171, 107, 235, 27, 155, 91, 219, 59, 187, 123, 251,
     7,  135, 71, 199, 39, 167, 103, 231, 23, 151, 87, 215, 55, 183, 119, 247,
-    15, 143, 79, 207, 47, 175, 111, 239, 31, 159, 95, 223, 63, 191, 127, 255};
+    15, 143, 79, 207, 47, 175, 111, 239, 31, 159, 95, 223, 63, 191, 127, 255
+};
 
 /* Unpack to "1" image */
 
@@ -718,6 +719,21 @@ ImagingUnpackBGRA15(UINT8 *out, const UINT8 *in, int pixels) {
     }
 }
 
+void
+ImagingUnpackBGRA15Z(UINT8 *out, const UINT8 *in, int pixels) {
+    int i, pixel;
+    /* RGB, rearranged channels, 5/5/5/1 bits per pixel, inverted alpha */
+    for (i = 0; i < pixels; i++) {
+        pixel = in[0] + (in[1] << 8);
+        out[B] = (pixel & 31) * 255 / 31;
+        out[G] = ((pixel >> 5) & 31) * 255 / 31;
+        out[R] = ((pixel >> 10) & 31) * 255 / 31;
+        out[A] = ~((pixel >> 15) * 255);
+        out += 4;
+        in += 2;
+    }
+}
+
 void
 ImagingUnpackRGB16(UINT8 *out, const UINT8 *in, int pixels) {
     int i, pixel;
@@ -790,6 +806,17 @@ ImagingUnpackBGRX(UINT8 *_out, const UINT8 *in, int pixels) {
     }
 }
 
+static void
+ImagingUnpackBGXR(UINT8 *_out, const UINT8 *in, int pixels) {
+    int i;
+    for (i = 0; i < pixels; i++) {
+        UINT32 iv = MAKE_UINT32(in[3], in[1], in[0], 255);
+        memcpy(_out, &iv, sizeof(iv));
+        in += 4;
+        _out += 4;
+    }
+}
+
 static void
 ImagingUnpackXRGB(UINT8 *_out, const UINT8 *in, int pixels) {
     int i;
@@ -856,7 +883,8 @@ unpackRGBa16L(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[1] * 255 / a),
                 CLIP8(in[3] * 255 / a),
                 CLIP8(in[5] * 255 / a),
-                a);
+                a
+            );
         }
         memcpy(_out, &iv, sizeof(iv));
         in += 8;
@@ -880,7 +908,8 @@ unpackRGBa16B(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[0] * 255 / a),
                 CLIP8(in[2] * 255 / a),
                 CLIP8(in[4] * 255 / a),
-                a);
+                a
+            );
         }
         memcpy(_out, &iv, sizeof(iv));
         in += 8;
@@ -904,7 +933,8 @@ unpackRGBa(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[0] * 255 / a),
                 CLIP8(in[1] * 255 / a),
                 CLIP8(in[2] * 255 / a),
-                a);
+                a
+            );
         }
         memcpy(_out, &iv, sizeof(iv));
         in += 4;
@@ -928,7 +958,8 @@ unpackRGBaskip1(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[0] * 255 / a),
                 CLIP8(in[1] * 255 / a),
                 CLIP8(in[2] * 255 / a),
-                a);
+                a
+            );
         }
         in += 5;
     }
@@ -950,7 +981,8 @@ unpackRGBaskip2(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[0] * 255 / a),
                 CLIP8(in[1] * 255 / a),
                 CLIP8(in[2] * 255 / a),
-                a);
+                a
+            );
         }
         in += 6;
     }
@@ -972,7 +1004,8 @@ unpackBGRa(UINT8 *_out, const UINT8 *in, int pixels) {
                 CLIP8(in[2] * 255 / a),
                 CLIP8(in[1] * 255 / a),
                 CLIP8(in[0] * 255 / a),
-                a);
+                a
+            );
         }
         memcpy(_out, &iv, sizeof(iv));
         in += 4;
@@ -1003,7 +1036,8 @@ unpackRGBAL(UINT8 *_out, const UINT8 *in, int pixels) {
             in[i],
             in[i + pixels],
             in[i + pixels + pixels],
-            in[i + pixels + pixels + pixels]);
+            in[i + pixels + pixels + pixels]
+        );
         memcpy(_out, &iv, sizeof(iv));
     }
 }
@@ -1090,6 +1124,17 @@ unpackBGRA16B(UINT8 *_out, const UINT8 *in, int pixels) {
     }
 }
 
+static void
+unpackBGAR(UINT8 *_out, const UINT8 *in, int pixels) {
+    int i;
+    for (i = 0; i < pixels; i++) {
+        UINT32 iv = MAKE_UINT32(in[3], in[1], in[0], in[2]);
+        memcpy(_out, &iv, sizeof(iv));
+        in += 4;
+        _out += 4;
+    }
+}
+
 /* Unpack to "CMYK" image */
 
 static void
@@ -1415,90 +1460,90 @@ band3I(UINT8 *out, const UINT8 *in, int pixels) {
 }
 
 static void
-band016B(UINT8* out, const UINT8* in, int pixels)
-{
+band016B(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 0 only, big endian */
     for (i = 0; i < pixels; i++) {
         out[0] = in[0];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band116B(UINT8* out, const UINT8* in, int pixels)
-{
+band116B(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 1 only, big endian */
     for (i = 0; i < pixels; i++) {
         out[1] = in[0];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band216B(UINT8* out, const UINT8* in, int pixels)
-{
+band216B(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 2 only, big endian */
     for (i = 0; i < pixels; i++) {
         out[2] = in[0];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band316B(UINT8* out, const UINT8* in, int pixels)
-{
+band316B(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 3 only, big endian */
     for (i = 0; i < pixels; i++) {
         out[3] = in[0];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band016L(UINT8* out, const UINT8* in, int pixels)
-{
+band016L(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 0 only, little endian */
     for (i = 0; i < pixels; i++) {
         out[0] = in[1];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band116L(UINT8* out, const UINT8* in, int pixels)
-{
+band116L(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 1 only, little endian */
     for (i = 0; i < pixels; i++) {
         out[1] = in[1];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band216L(UINT8* out, const UINT8* in, int pixels)
-{
+band216L(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 2 only, little endian */
     for (i = 0; i < pixels; i++) {
         out[2] = in[1];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
 static void
-band316L(UINT8* out, const UINT8* in, int pixels)
-{
+band316L(UINT8 *out, const UINT8 *in, int pixels) {
     int i;
     /* band 3 only, little endian */
     for (i = 0; i < pixels; i++) {
         out[3] = in[1];
-        out += 4; in += 2;
+        out += 4;
+        in += 2;
     }
 }
 
@@ -1516,7 +1561,7 @@ static struct {
 
     /* flags: "I" inverted data; "R" reversed bit order; "B" big
        endian byte order (default is little endian); "L" line
-       interleave, "S" signed, "F" floating point */
+       interleave, "S" signed, "F" floating point, "Z" inverted alpha */
 
     /* exception: rawmodes "I" and "F" are always native endian byte order */
 
@@ -1560,6 +1605,7 @@ static struct {
     {"P", "P", 8, copy1},
     {"P", "P;R", 8, unpackLR},
     {"P", "L", 8, copy1},
+    {"P", "PX", 16, unpackL16B},
 
     /* palette w. alpha */
     {"PA", "PA", 16, unpackLA},
@@ -1577,13 +1623,18 @@ static struct {
     {"RGB", "BGR;15", 16, ImagingUnpackBGR15},
     {"RGB", "RGB;16", 16, ImagingUnpackRGB16},
     {"RGB", "BGR;16", 16, ImagingUnpackBGR16},
+    {"RGB", "RGBX;16L", 64, unpackRGBA16L},
+    {"RGB", "RGBX;16B", 64, unpackRGBA16B},
     {"RGB", "RGB;4B", 16, ImagingUnpackRGB4B},
     {"RGB", "BGR;5", 16, ImagingUnpackBGR15}, /* compat */
     {"RGB", "RGBX", 32, copy4},
     {"RGB", "RGBX;L", 32, unpackRGBAL},
+    {"RGB", "RGBXX", 40, copy4skip1},
+    {"RGB", "RGBXXX", 48, copy4skip2},
     {"RGB", "RGBA;L", 32, unpackRGBAL},
     {"RGB", "RGBA;15", 16, ImagingUnpackRGBA15},
     {"RGB", "BGRX", 32, ImagingUnpackBGRX},
+    {"RGB", "BGXR", 32, ImagingUnpackBGXR},
     {"RGB", "XRGB", 32, ImagingUnpackXRGB},
     {"RGB", "XBGR", 32, ImagingUnpackXBGR},
     {"RGB", "YCC;P", 24, ImagingUnpackYCC},
@@ -1618,12 +1669,14 @@ static struct {
     {"RGBA", "RGBA;L", 32, unpackRGBAL},
     {"RGBA", "RGBA;15", 16, ImagingUnpackRGBA15},
     {"RGBA", "BGRA;15", 16, ImagingUnpackBGRA15},
+    {"RGBA", "BGRA;15Z", 16, ImagingUnpackBGRA15Z},
     {"RGBA", "RGBA;4B", 16, ImagingUnpackRGBA4B},
     {"RGBA", "RGBA;16L", 64, unpackRGBA16L},
     {"RGBA", "RGBA;16B", 64, unpackRGBA16B},
     {"RGBA", "BGRA", 32, unpackBGRA},
     {"RGBA", "BGRA;16L", 64, unpackBGRA16L},
     {"RGBA", "BGRA;16B", 64, unpackBGRA16B},
+    {"RGBA", "BGAR", 32, unpackBGAR},
     {"RGBA", "ARGB", 32, unpackARGB},
     {"RGBA", "ABGR", 32, unpackABGR},
     {"RGBA", "YCCA;P", 32, ImagingUnpackYCCA},
@@ -1642,6 +1695,7 @@ static struct {
 
 #ifdef WORDS_BIGENDIAN
     {"RGB", "RGB;16N", 48, unpackRGB16B},
+    {"RGB", "RGBX;16N", 64, unpackRGBA16B},
     {"RGBA", "RGBa;16N", 64, unpackRGBa16B},
     {"RGBA", "RGBA;16N", 64, unpackRGBA16B},
     {"RGBX", "RGBX;16N", 64, unpackRGBA16B},
@@ -1655,6 +1709,7 @@ static struct {
     {"RGBA", "A;16N", 16, band316B},
 #else
     {"RGB", "RGB;16N", 48, unpackRGB16L},
+    {"RGB", "RGBX;16N", 64, unpackRGBA16L},
     {"RGBA", "RGBa;16N", 64, unpackRGBa16L},
     {"RGBA", "RGBA;16N", 64, unpackRGBA16L},
     {"RGBX", "RGBX;16N", 64, unpackRGBA16L},
@@ -1662,7 +1717,6 @@ static struct {
     {"RGB", "G;16N", 16, band116L},
     {"RGB", "B;16N", 16, band216L},
 
-
     {"RGBA", "R;16N", 16, band016L},
     {"RGBA", "G;16N", 16, band116L},
     {"RGBA", "B;16N", 16, band216L},
